@@ -56,6 +56,7 @@ tlist *int_terms;	/* T the set of all inter-module terminals */
 /* FOR THE ROUTER (only) */
 int currentPartition;
 tile **routingRoot[2];	  
+
 /* NOT USED ENOUGH */
 int lcount = 0;			   /* number of input lines read */
 int module_count = 0;		   /* number of modules/objects read in */
@@ -72,7 +73,7 @@ int debug_level = 0;			      	/* -d qualifier, sets the debug level */
 int stopAtFirstCut = FALSE;			/* -f flag; terminates GR early */
 int congestion_rule = 2; 			/* -g qualifier, used for conGestion */
 int do_hand_placement = FALSE;			/* -h flag */
-int includeSysterms = TRUE;			/* -i flag; excludes system terminals */
+int includeSysterms = FALSE;			/* -i flag; excludes system terminals */
 
 int cc_search_level = CC_SEARCH_LEVEL;		/* -l qualifier, limits cc search */
 int matrix_type = UNSIGNED;			/* -m qualifier, sets signal-flow style*/
@@ -87,7 +88,6 @@ int useAveraging = FALSE;			/* -w flag; turns Weighted averaging on*/
 int latex = FALSE;				/* -x flag; modified PostScript header */
 int recordTiming = FALSE;			/* -y flag; turns time collection on */
 int xcircuit= TRUE;				/* -z flag;to launch spar into xcircuit*/
-char *ivffilename; 			      	/* input file name */
 FILE *stats;					/* quality statistics file */
 
 /* NOT USED MUCH */
@@ -123,7 +123,7 @@ void GenerateXcircuitFormat(Clientdata *areastruct, FILE *ps,Boolean bIsSparmode
  */
 
 
-    FILE *ivffile, *timingFile, *fopen();
+    FILE *timingFile, *fopen();
     //collapseNullModules = FALSE;
     int c,  errflg = 0, x1, y1, x2, y2;
     double atof();
@@ -144,17 +144,7 @@ void GenerateXcircuitFormat(Clientdata *areastruct, FILE *ps,Boolean bIsSparmode
     collapseNullModules=FALSE;
     useBlockPartitioning=FALSE;
     outputFile = stdout;	/* -o qualifier, default behavior: opens stdout for output */
-    argc=1;
-    argv[0]="test1.ivf";
 
-  //  ivffilename = strdup(argv[optind]);
-    
-    if (ps == NULL) {
-	fprintf(stderr,"can't open %s\n", "dfgfd");
-	exit(1);
-    }
-    else fprintf(stderr,"%s, used for ivffile\n", "ddfgfd");
-    
     /*
      * main body
      */
@@ -168,8 +158,8 @@ void GenerateXcircuitFormat(Clientdata *areastruct, FILE *ps,Boolean bIsSparmode
 	    fprintf(stderr,"can't open file for timing results\n");
 	    exit(1);
 	}
-	fprintf(timingFile,"Timing results for %s with flags -p%d -s%d -c%d -m%d -l%d -d%d %s %s %s %s\n\n",
-		ivffilename, partition_rule, max_partition_size, max_partition_conn, 
+	fprintf(timingFile,"Timing results with flags -p%d -s%d -c%d -m%d -l%d -d%d %s %s %s %s\n\n",
+		partition_rule, max_partition_size, max_partition_conn, 
 		matrix_type, cc_search_level, debug_level,
 		(do_routing == FALSE) ? "-n" : (stopAtFirstCut == TRUE) ? "-f" : " ",
 		(useSlivering == FALSE) ? "-v" : " ",
@@ -180,10 +170,7 @@ void GenerateXcircuitFormat(Clientdata *areastruct, FILE *ps,Boolean bIsSparmode
 	mark_time(timingFile, "startup", &diffTime);
     }
 
-    read_ivf(ps);  
-    if(recordTiming == TRUE) mark_time(timingFile, "parse", &diffTime);
-
-    /* place */
+     /* place */
     if (do_hand_placement == FALSE)
     {
 	fprintf(stderr,"Placing internal modules...");
@@ -493,7 +480,7 @@ Route()
 void Route(Clientdata *areastruct,Boolean bIsSparmode) 
 {
 
-    FILE *ivffile, *timingFile, *fopen();
+    FILE *timingFile, *fopen();
     //collapseNullModules = FALSE;
     int c,  errflg = 0, x1, y1, x2, y2;
     double atof();
@@ -516,7 +503,6 @@ void Route(Clientdata *areastruct,Boolean bIsSparmode)
     outputFile = stdout;
 
     argc=1;
-    argv[0]="test1.ivf";
 
     if(recordTiming == TRUE) mark_time(timingFile, "parse", &diffTime);
 
