@@ -30,8 +30,10 @@
 #include <string.h>
 #include "system_defs.h"
 #include "net.h"
-#include "ivf_format.h"
+#include "network.h"
 #include "externs.h"
+#include "psfigs.h"
+#include "asg_module.h"
 
 /*---------------------------------------------------------------
  * Forward References
@@ -347,13 +349,11 @@ module *newobject(gate_name, gate_type)
     object->index = module_count++;
     object->name = strdup(gate_name);
     object->type = strdup(gate_type);
-    object->dly_type = INERTIAL_DELAY;
-    object->delay = 0;
     object->x_pos = 0;
     object->y_pos = 0;    
     object->flag = UNSEEN;
     object->placed = UNPLACED;
-    object->rot = LEFT;		/* asuume no rotation */
+    object->rot = LEFT;		/* assme no rotation */
     object->primary_in = NULL;
     object->primary_out = NULL;
     object->terms = NULL;
@@ -510,7 +510,6 @@ void add_in_out(name, index, noOfTerms, dir)
     /* find this net in the global database */
     tn = get_net(name);
     tt->nt = tn;    
-
     tt->mod = curobj_module;
     tt->name = (char *)calloc(1, TERM_NAME_SIZE);
 
@@ -810,7 +809,7 @@ int fix_pin_position(m, nn, x, y)
 }
 
 /* ------------------------------------------------------------------------
- * reset_default_icon_size - use the x,y dimensions from .ivf   added 3-92 (stf)
+ * reset_default_icon_size - use the x,y dimensions  added 3-92 (stf)
  *-------------------------------------------------------------------------
  */
 int reset_default_icon_size(m, x, y)
@@ -974,16 +973,7 @@ int reposition_all_terminals(m, incount, inoutcount, outcount, spacesPerPin)
     }
 }
 
-/* ------------------------------------------------------------------------
- * adddelay: add the simulation delay information to a module
- * ------------------------------------------------------------------------
- */
-void adddelay(time)
-    int time;
-{
-    curobj_module->delay = time;
-}
-    
+   
 /* ------------------------------------------------------------------------
  * add_port: add a terminal to the list of system terminals
  * this list is different from the other terminal lists:
@@ -1377,6 +1367,30 @@ void clip_null_gates()
     free_list(temp);
 }
 
+/*---------------------------------------------------------------
+ * str_end_copy
+ *---------------------------------------------------------------
+ */
+
+char *str_end_copy(new, old, n)
+    char *new, *old;
+    int n;
+{
+    /* Copy the last <n> characters from <old> to <new>. */
+    int i, len = strlen(old);
+    if (len > n)
+    {
+	strncpy(new, old, n);
+	for (i=0; i<n; i++)
+	{
+	    new[i] = old[len-n+i];
+	}
+	new[n] = '\0';
+    }
+    else strcpy(new, old);
+
+    return(new);
+}
 
 /* ------------------------------------------------------------------------
  * END OF FILE
