@@ -1303,8 +1303,10 @@ void centerview(objinstptr tinst)
 void refresh(xcWidget bw, caddr_t clientdata, caddr_t calldata)
 {
    drawarea(NULL, NULL, NULL);
-   drawhbar(areastruct.scrollbarh, NULL, NULL);
-   drawvbar(areastruct.scrollbarv, NULL, NULL);
+   if (areastruct.scrollbarh)
+      drawhbar(areastruct.scrollbarh, NULL, NULL);
+   if (areastruct.scrollbarv)
+      drawvbar(areastruct.scrollbarv, NULL, NULL);
    printname(topobject);
 }
 
@@ -1863,6 +1865,8 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
      for (areagen = theobject->plist; areagen < theobject->plist +
            theobject->parts; areagen++) {
 
+       if ((*areagen)->type & DRAW_HIDE) continue;
+
        if (defaultcolor != DOFORALL) {
 	  if ((*areagen)->color != curcolor) {
 	     if ((*areagen)->color == DEFAULTCOLOR)
