@@ -47,6 +47,7 @@
 //-----------------------------------------------------------------------------
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <string.h>
 #include <map>
 #include <vector>
@@ -56,9 +57,9 @@ using namespace std;
 #include "utils.hh"
 #include "device.hh"
 
-extern "C" { void Add2TermModule( char *,char *,char *,char * ); }
-extern "C" { void Add3TermModule( char *,char *,char *,char *, char *); }
-extern "C" { void Add4TermModule( char *,char *,char *,char *, char *, char *);}
+#define _USING_CC
+#include "../../asg/asg_parse_support.h"
+#undef _USING_CC
 
 // ****************************************************************************
 // class: device
@@ -480,6 +481,13 @@ void mosfet::print(ostream& os) {
 	char* pNode4 = pNet4Ref->name();
 	// This is not quite right.  Sometimes you'll want a four-terminal module here -STF
 	Add3TermModule( pName, "MSFET" , pNode1, pNode2, pNode3);
+
+//	if (_bulk && _bulk->net_ref()) {
+//	    AddNTermModule( pName, "MSFET" , 4, "D", pNode1, "G", pNode2, "S", pNode3,
+//			"B", pNode4);
+//	else
+//	    AddNTermModule( pName, "MSFET" , 3, "D", pNode1, "G", pNode2, "S", pNode3); */
+
 }
 void diode::print(ostream& os) {
 
@@ -491,6 +499,9 @@ void diode::print(ostream& os) {
 */
 	char* pName = name();
 	Add2TermModule( pName, "RESTR" , (char*)(_nplus), (char*)(_nminus));
+
+//	AddNTermModule( pName, "RESTR" , 2, "1", (char*)(_nplus), "2",
+//		(char*)(_nminus));
 }
 void resistor::print(ostream& os) {
 
@@ -510,6 +521,7 @@ void resistor::print(ostream& os) {
         
 
 	Add2TermModule( pName, "RESTR" , pNode1, pNode2);
+//	AddNTermModule( pName, "RESTR" , 2, "1", pNode1, "2", pNode2);
 }
 void capacitor::print(ostream& os) {
 
@@ -528,6 +540,7 @@ void capacitor::print(ostream& os) {
 	char* pNode2 = pNet2Ref->name();
 
 	Add2TermModule( pName, "CAPC" , pNode1, pNode2 );
+//	AddNTermModule( pName, "CAPC" , 2, "1", pNode1, "2", pNode2 );
 }
 void coupling::print(ostream& os) {
 
@@ -540,6 +553,9 @@ void coupling::print(ostream& os) {
 	char* pName = name();
 	Add2TermModule( pName, "RESTR" , (char*)(_inductor1), (char*)(_inductor2));
 
+//	AddNTermModule( pName, "RESTR" , 2, "1", (char*)(_inductor1),
+//		"2", (char*)(_inductor2));
+
 }
 void inductor::print(ostream& os) {
 /*
@@ -559,6 +575,8 @@ void inductor::print(ostream& os) {
 
 	Add2TermModule( pName, "INDR" , pNode1, pNode2 );
 
+//	AddNTermModule( pName, "INDR" , 2, "1", pNode1, "2", pNode2 );
+
 }
 void vsource::print(ostream& os) {
 /*
@@ -577,6 +595,8 @@ void vsource::print(ostream& os) {
 
 
 	Add2TermModule( pName, "VAMP" , pNode1, pNode2 );
+
+//	AddNTermModule( pName, "VAMP" , 2, "P", pNode1, "M", pNode2 );
 }
 void isource::print(ostream& os) {
 /*
@@ -597,6 +617,7 @@ void isource::print(ostream& os) {
 
 	Add2TermModule( pName, "VAMP" , pNode1, pNode2 );
 
+//	AddNTermModule( pName, "VAMP" , 2, "P", pNode1, "M", pNode2 );
 }
 
 
