@@ -8,8 +8,8 @@
 /*-------------------------------------------------------------------------*/
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -1242,9 +1242,11 @@ void copycat()
       }
 
       (*newobj)->schemtype = oldobj->schemtype;
-      (*newobj)->netlist = NULL;
-      (*newobj)->portlist = NULL;
-      (*newobj)->calllist = NULL;
+      (*newobj)->netnames = NULL;
+      (*newobj)->ports = NULL;
+      (*newobj)->calls = NULL;
+      (*newobj)->polygons = NULL;
+      (*newobj)->labels = NULL;
       (*newobj)->valid = False;
       (*newobj)->traversed = False;
       (*newobj)->devname = NULL;
