@@ -373,7 +373,7 @@ void settsize(xcWidget w, labelptr settext)
 	    redrawtext(nstext);
 	 }
       }
-      objectdeselect();
+      unselect_all();
    }
 }
 
@@ -832,12 +832,12 @@ void setosize(xcWidget w, objinstptr dataptr)
    }
    for (osel = areastruct.selectlist; osel < areastruct.selectlist +
 	     areastruct.selects; osel++) {
-      if (SELECTTYPE(osel) == OBJECT) {
+      if (SELECTTYPE(osel) == OBJINST) {
 	 nsobj = SELTOOBJINST(osel);
          nsobj->scale = tmpres;
       }
    }
-   objectdeselect();
+   unselect_all();
    pwriteback(areastruct.topinstance);
    drawarea(NULL, NULL, NULL);
 }
@@ -856,7 +856,7 @@ void getosize(xcWidget button, caddr_t clientdata, caddr_t calldata)
    objinstptr setobj = NULL;
 
    for (; osel < areastruct.selectlist + areastruct.selects; osel++)
-      if (SELECTTYPE(osel) == OBJECT) {
+      if (SELECTTYPE(osel) == OBJINST) {
 	 setobj = SELTOOBJINST(osel);
 	 selects++;
 	 break;
@@ -929,7 +929,7 @@ void setwwidth(xcWidget w, void *dataptr)
             nspath->width = tmpres;
          }
       }
-      objectdeselect();
+      unselect_all();
       pwriteback(areastruct.topinstance);
       drawarea(NULL, NULL, NULL);
    }
@@ -949,19 +949,19 @@ void getwwidth(xcWidget button, caddr_t clientdata, caddr_t calldata)
 
    for (; osel < areastruct.selectlist + areastruct.selects; osel++) {
       setel = *(topobject->plist + (*osel));
-      if (setel->type == ARC) {
+      if (IS_ARC(setel)) {
 	 flval = ((arcptr)setel)->width;
 	 break;
       }
-      else if (setel->type == POLYGON) {
+      else if (IS_POLYGON(setel)) {
 	 flval = ((polyptr)setel)->width;
 	 break;
       }
-      else if (setel->type == SPLINE) {
+      else if (IS_SPLINE(setel)) {
 	 flval = ((splineptr)setel)->width;
 	 break;
       }
-      else if (setel->type == PATH) {
+      else if (IS_PATH(setel)) {
 	 flval = ((pathptr)setel)->width;
 	 break;
       }
@@ -1823,7 +1823,7 @@ polyptr checkforbbox(objectptr localdata)
 
    for (cbbox = (polyptr *)localdata->plist; cbbox <
 	(polyptr *)localdata->plist + localdata->parts; cbbox++)
-      if ((*cbbox)->type == POLYGON)
+      if (IS_POLYGON(*cbbox))
          if ((*cbbox)->style & BBOX) return *cbbox;
 
    return NULL;
@@ -1862,7 +1862,7 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
       for (sstyle = areastruct.selectlist; sstyle < areastruct.selectlist
 	   + areastruct.selects; sstyle++) {
 	 short stype = SELECTTYPE(sstyle);
-	 if (stype == ARC || stype == POLYGON || stype == SPLINE || stype == PATH) {
+	 if (stype & (ARC | POLYGON | SPLINE | PATH)) {
 	    short *estyle;
 	    switch (stype) {
 	       case ARC:
@@ -1925,7 +1925,7 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
    }
 #ifndef TCL_WRAPPER
    setallstylemarks(newstyle);
-   objectdeselect();
+   unselect_all();
 #endif
    return (int)newstyle;
 }
@@ -2056,7 +2056,7 @@ void setcolor(xcWidget w, pointertype value, caddr_t calldata)
    }
 
 #ifndef TCL_WRAPPER
-   objectdeselect();
+   unselect_all();
 #endif
 }
 
@@ -2309,7 +2309,7 @@ void stringparam(xcWidget w, caddr_t clientdata, caddr_t calldata)
       if (eventmode == TEXT2_MODE || eventmode == TEXT3_MODE) {
          settext = (genericptr *)EDITPART;
          makeparam(TOLABEL(settext), _STR2);
-         objectdeselect();
+         unselect_all();
 	 setparammarks(NULL);
       }
       else if (checkselect(LABEL)) parameterize(P_SUBSTRING, _STR2);
@@ -2353,7 +2353,7 @@ void startunparam(xcWidget w, pointertype value, caddr_t calldata)
    }
    else
       unparameterize((int)value);
-   objectdeselect();
+   unselect_all();
    setparammarks(NULL);
 }
 
@@ -2479,7 +2479,7 @@ void setvjust(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setjust(w, value, NULL, 2);
-      else objectdeselect();
+      else unselect_all();
    }
 }
 
@@ -2508,7 +2508,7 @@ void sethjust(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setjust(w, value, NULL, 1);
-      else objectdeselect();
+      else unselect_all();
    }
 }
 
@@ -2537,7 +2537,7 @@ void setjustbit(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) dojustifybit(w, NULL, (short)value);
-      else objectdeselect();
+      else unselect_all();
    }
 }
 
@@ -2565,7 +2565,7 @@ void setpinjustbit(xcWidget w, pointertype value, caddr_t nulldata)
                dopinjustbit(NULL, settext, (short)value);
          }
       }
-      objectdeselect();
+      unselect_all();
    }
 }
 
@@ -2831,7 +2831,7 @@ void setfont(xcWidget w, pointertype value, caddr_t calldata)
          }
       }
       if (labelcount == 0) setfontval(w, value, NULL);
-      else objectdeselect();
+      else unselect_all();
    }
 }
 
@@ -2927,7 +2927,7 @@ void fontstyle(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontstyle(w, value, NULL);
-      else objectdeselect();
+      else unselect_all();
    }
 }
 
@@ -3022,7 +3022,7 @@ void fontencoding(xcWidget w, pointertype value, caddr_t nulldata)
          }
       }
       if (labelcount == 0) setfontencoding(w, value, NULL);
-      else objectdeselect();
+      else unselect_all();
    }
 }
 
