@@ -52,21 +52,22 @@ extern char _STR[150];
 /*----------------------------------------------------------------------*/
 
 u_char param_select[] = {
-   SEL_ANY,					/* P_NUMERIC */
+   ALL_TYPES,					/* P_NUMERIC */
    LABEL,					/* P_SUBSTRING */
-   POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION_X */
-   POLYGON | SPLINE | LABEL | OBJECT | ARC,	/* P_POSITION_Y */
+   POLYGON | SPLINE | LABEL | OBJINST | ARC,	/* P_POSITION_X */
+   POLYGON | SPLINE | LABEL | OBJINST | ARC,	/* P_POSITION_Y */
    POLYGON | SPLINE | ARC | PATH,		/* P_STYLE */
    LABEL,					/* P_JUSTIFY */
    ARC,						/* P_ANGLE1 */
    ARC,						/* P_ANGLE2 */
    ARC,						/* P_RADIUS */
    ARC,						/* P_MINOR_AXIS */
-   LABEL | OBJECT,				/* P_ROTATION */
-   LABEL | OBJECT,				/* P_SCALE */
+   LABEL | OBJINST,				/* P_ROTATION */
+   LABEL | OBJINST,				/* P_SCALE */
    POLYGON | SPLINE | ARC | PATH,		/* P_LINEWIDTH */
-   SEL_ANY, 					/* P_COLOR (tbd) */
-   POLYGON | SPLINE | LABEL | OBJECT | ARC 	/* P_POSITION */
+   ALL_TYPES, 					/* P_COLOR */
+   ALL_TYPES, 					/* P_EXPRESSION */
+   POLYGON | SPLINE | LABEL | OBJINST | ARC 	/* P_POSITION */
 };
 
 #ifdef TCL_WRAPPER
@@ -243,7 +244,7 @@ void free_element_param(genericptr thiselem, eparamptr thisepp)
 	 /* stored as an allocated string in pdata.refkey, which needs	*/
 	 /* to be free'd.						*/
 
-	 if ((thiselem->type == OBJECT) && (epp->pdata.refkey != NULL))
+	 if (IS_OBJINST(thiselem) && (epp->pdata.refkey != NULL))
 	    free(epp->pdata.refkey);
 
 	 free(epp->key);
@@ -277,6 +278,21 @@ void free_instance_param(objinstptr thisinst, oparamptr thisparam)
    }
 }
 
+/*----------------------------------------------------------------------*/
+/* Convenience function used by files.c to set a color parameter.	*/
+/*----------------------------------------------------------------------*/
+
+void std_eparam(genericptr gen, char *key)
+{
+   eparamptr epp;
+
+   if (key == NULL) return;
+
+   epp = make_new_eparam(key);
+   epp->next = gen->passed;
+   gen->passed = epp;
+}
+
 /*----------------------------------------------*/
 /* Draw a circle at all parameter positions	*/
 /*----------------------------------------------*/
@@ -301,7 +317,7 @@ void indicateparams(genericptr thiselem)
 		  case LABEL:
 	             UDrawCircle(&TOLABEL(&thiselem)->position, ops->which);
 		     break;
-		  case OBJECT:
+		  case OBJINST:
 	             UDrawCircle(&TOOBJINST(&thiselem)->position, ops->which);
 		     break;
 		  case POLYGON:
@@ -413,7 +429,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
    int i, j, k, ival;
    char *key;
    float fval;
-   oparamptr ops, ips;
+   oparamptr dps, ops;
    eparamptr epp;
    XPoint *setpt;
    Boolean needrecalc;	/* for arcs and splines */
@@ -428,34 +444,36 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 
       for (epp = thiselem->passed; epp != NULL; epp = epp->next) {
 
-	 ips = (pinst != NULL) ?  find_param(pinst, epp->key) : NULL;
-	 ops = match_param(thisobj, epp->key);
+	 /* Use the parameter from the instance, if available.	*/
+	 /* Otherwise, revert to the type of the object.	*/
+	 /* Normally they will be the same.			*/
 
-	 if (ips != NULL) {
+         ops = match_param(thisobj, epp->key);
+	 dps = (pinst != NULL) ?  find_param(pinst, epp->key) : ops;
+
+	 if (dps != NULL) {
 
 	    /* Get integer and float values.  Promote types if necessary */
 
-	    switch(ips->type) {
+	    switch(dps->type) {
 	       case XC_INT:
-		  ival = ips->parameter.ivalue;
+		  ival = dps->parameter.ivalue;
 		  fval = (float)(ival);
 		  break;
 	       case XC_FLOAT:
-		  fval = ips->parameter.fvalue;
+		  fval = dps->parameter.fvalue;
 		  ival = (int)(fval + 0.5);
 		  break;
 	       case XC_STRING:
-		  promoted = textprint(ips->parameter.string, pinst);
+		  promoted = textprint(dps->parameter.string, pinst);
 		  if (sscanf(promoted, "%g", &fval) == 1)
 		     ival = (int)(fval + 0.5);
-		  else {
-		     free(promoted);
-		     continue;
-		  }
+		  else
+		     ival = 0;
 		  free(promoted);
 		  break;
 	       case XC_EXPR:
-		  if ((promoted = evaluate_expr(ips)) == NULL) continue;
+		  if ((promoted = evaluate_expr(dps)) == NULL) continue;
 		  if (sscanf(promoted, "%g", &fval) == 1)
 		     ival = (int)(fval + 0.5);
 		  free(promoted);
@@ -465,7 +483,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	 else if (ops == NULL)
 	    continue;
 
-	 if (thiselem->type == OBJECT) {
+	 if (IS_OBJINST(thiselem)) {
 	    key = epp->pdata.refkey;
 	    if (key != NULL) {
 	       objinstptr thisinst;
@@ -476,7 +494,8 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	       /* Sanity check: refkey must exist in object */
 	       refop = match_param(thisinst->thisobject, key);
 	       if (refop == NULL) {
-		  Fprintf("Error:  Reference key %s does not exist in object %s\n",
+		  Fprintf(stderr, "Error:  Reference key %s does not"
+				" exist in object %s\n",
 				key, thisinst->thisobject->name);
 		  continue;
 	       }
@@ -487,7 +506,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		  free_instance_param(thisinst, newop);
 	       
 	       /* Create a new instance parameter */
-	       newop = copyparameter((ips != NULL) ? ips : ops);
+	       newop = copyparameter(dps);
 	       newop->next = thisinst->params;
 	       thisinst->params = newop;
 
@@ -516,7 +535,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	          case LABEL:
 		     TOLABEL(&thiselem)->position.x = ival;
 		     break;
-	          case OBJECT:
+	          case OBJINST:
 		     TOOBJINST(&thiselem)->position.x = ival;
 		     break;
 	          case ARC:
@@ -538,7 +557,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	          case LABEL:
 		     TOLABEL(&thiselem)->position.y = ival;
 		     break;
-	          case OBJECT:
+	          case OBJINST:
 		     TOOBJINST(&thiselem)->position.y = ival;
 		     break;
 	          case ARC:
@@ -614,7 +633,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	          case LABEL:
 		     TOLABEL(&thiselem)->rotation = ival;
 		     break;
-	          case OBJECT:
+	          case OBJINST:
 		     TOOBJINST(&thiselem)->rotation = ival;
 		     break;
 	       }
@@ -625,7 +644,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 	          case LABEL:
 		     TOLABEL(&thiselem)->scale = fval;
 		     break;
-	          case OBJECT:
+	          case OBJINST:
 		     TOOBJINST(&thiselem)->scale = fval;
 		     break;
 	       }
@@ -647,6 +666,10 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		     break;
 	       }
 	       break;
+	    case P_COLOR:
+	       retval = max(retval, 0);
+	       thiselem->color = ival;
+	       break;
          }
       }
       /* substitutions into arcs and splines require that the	*/
@@ -691,7 +714,7 @@ int psubstitute(objinstptr thisinst)
 
 Boolean has_param(genericptr celem)
 {
-   if (celem->type == LABEL) {
+   if (IS_LABEL(celem) == LABEL) {
       stringpart *cstr;
       labelptr clab = (labelptr)celem;
       for (cstr = clab->string; cstr != NULL; cstr = cstr->nextpart)
@@ -756,7 +779,7 @@ void pwriteback(objinstptr thisinst)
                switch(ops->which) {
 	          case P_POSITION_X:
 	             switch(thiselem->type) {
-	                case OBJECT:
+	                case OBJINST:
 		           wtemp.ival = TOOBJINST(&thiselem)->position.x;
 		           break;
 	                case LABEL:
@@ -776,7 +799,7 @@ void pwriteback(objinstptr thisinst)
 	             break;
 	          case P_POSITION_Y:
 	             switch(thiselem->type) {
-	                case OBJECT:
+	                case OBJINST:
 		           wtemp.ival = TOOBJINST(&thiselem)->position.y;
 		           break;
 	                case LABEL:
@@ -847,7 +870,7 @@ void pwriteback(objinstptr thisinst)
 	             break;
 	          case P_ROTATION:
 	             switch(thiselem->type) {
-	                case OBJECT:
+	                case OBJINST:
 		           wtemp.ival = TOOBJINST(&thiselem)->rotation;
 		           break;
 	                case LABEL:
@@ -857,7 +880,7 @@ void pwriteback(objinstptr thisinst)
 	             break;
 	          case P_SCALE:
 	             switch(thiselem->type) {
-	                case OBJECT:
+	                case OBJINST:
 		           wtemp.fval = TOOBJINST(&thiselem)->scale;
 		           break;
 	                case LABEL:
@@ -881,6 +904,9 @@ void pwriteback(objinstptr thisinst)
 		           break;
 	             }
 	             break;
+		  case P_COLOR:
+		     wtemp.ival = thiselem->color;
+		     break;
                }
 	       destivalptr = (ips != NULL) ? &ips->parameter.ivalue
 			: &ops->parameter.ivalue;
@@ -1041,7 +1067,7 @@ eparamptr copyeparam(eparamptr cepp, genericptr thiselem)
    eparamptr newepp;
 
    newepp = make_new_eparam(cepp->key);
-   if ((thiselem->type == OBJECT) && (cepp->pdata.refkey != NULL))
+   if (IS_OBJINST(thiselem) && (cepp->pdata.refkey != NULL))
       newepp->pdata.refkey = strdup(cepp->pdata.refkey);
    else
       newepp->pdata.pointno = cepp->pdata.pointno;
@@ -1126,20 +1152,26 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
 void makeexprparam(objectptr refobject, char *key, char *value, int which)
 {
    oparamptr newops;
+   char *newkey;
+
+   /* Check against postscript reserved names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
 
    /* Ensure that no two parameters have the same name! */
 
-   if (check_param(refobject, key)) {
+   if (check_param(refobject, newkey)) {
       Wprintf("There is already a parameter with that key!");
    }
 
-   newops = make_new_parameter(key);
+   newops = make_new_parameter(newkey);
    newops->next = refobject->params;
    refobject->params = newops;
    newops->type = XC_EXPR;		/* expression requiring evaluation */
    newops->which = which;
    newops->parameter.expr = strdup(value);
    incr_changes(refobject);
+   free(newkey);
 }
 
 /*------------------------------------------------------------------*/
@@ -1149,10 +1181,15 @@ void makeexprparam(objectptr refobject, char *key, char *value, int which)
 void makefloatparam(objectptr refobject, char *key, float value)
 {
    oparamptr newops;
+   char *newkey;
+
+   /* Check against postscript reserved names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
 
    /* Ensure that no two parameters have the same name! */
 
-   if (check_param(refobject, key)) {
+   if (check_param(refobject, newkey)) {
       Wprintf("There is already a parameter with that key!");
    }
 
@@ -1163,6 +1200,7 @@ void makefloatparam(objectptr refobject, char *key, float value)
    newops->which = P_NUMERIC;
    newops->parameter.fvalue = value;
    incr_changes(refobject);
+   free(newkey);
 }
 
 /*----------------------------------------------------------------*/
@@ -1172,20 +1210,26 @@ void makefloatparam(objectptr refobject, char *key, float value)
 void makestringparam(objectptr refobject, char *key, stringpart *strptr)
 {
    oparamptr newops;
+   char *newkey;
+
+   /* Check against postscript reserved names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
 
    /* Ensure that no two parameters have the same name! */
 
-   if (check_param(refobject, key)) {
+   if (check_param(refobject, newkey)) {
       Wprintf("There is already a parameter with that key!");
    }
 
-   newops = make_new_parameter(key);
+   newops = make_new_parameter(newkey);
    newops->next = refobject->params;
    refobject->params = newops;
    newops->type = XC_STRING;
    newops->which = P_SUBSTRING;
    newops->parameter.string = strptr;
    incr_changes(refobject);
+   free(newkey);
 }
 
 /*--------------------------------------------------------------*/
@@ -1217,7 +1261,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 
    /* Parameterized strings are handled by makeparam() */
 
-   if ((*gelem)->type == LABEL && mode == P_SUBSTRING) {
+   if (IS_LABEL(*gelem) && mode == P_SUBSTRING) {
       Fprintf(stderr, "Error: String parameter passed to makenumericalp()\n");
       return;
    }
@@ -1241,7 +1285,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
    /* Ensure that no two parameters have the same name! */
 
    if (key) {
-      keyptr = key;
+      keyptr = checkpostscriptname(key, NULL, NULL);
    }
    else {
       strcpy(new_key, param_keys[mode]);
@@ -1277,7 +1321,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	 free_element_param(*gelem, epp);
 	 Fprintf(stderr, "Error: Attempt to link a parameter to "
 			"a parameter of a different type\n");
-	 return;
+	 goto param_done;
       }
       if ((newops = match_instance_param(areastruct.topinstance, keyptr)) == NULL) {
          newops = make_new_parameter(keyptr);
@@ -1293,11 +1337,14 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	 /* should redraw.						*/
          drawarea(NULL, NULL, NULL);
          newops = NULL;
-	 return;
+	 goto param_done;
       }
    }
 
    if (newops) {
+      if (mode == P_COLOR)
+	 newops->parameter.ivalue = (int)((*gelem)->color);
+
       switch((*gelem)->type) {
          case LABEL:
 	    switch(mode) {
@@ -1350,7 +1397,7 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	          break;
 	    }
 	    break;
-         case OBJECT:
+         case OBJINST:
 	    switch(mode) {
 	       case P_POSITION_X:
 	          newops->parameter.ivalue = (int)TOOBJINST(gelem)->position.x;
@@ -1422,6 +1469,9 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
 	    break;
       }
    }
+
+param_done:
+   if (keyptr != new_key) free(keyptr);
 }
 
 /*--------------------------------------------------------------*/
@@ -1573,6 +1623,7 @@ void makeparam(labelptr thislabel, char *key)
 {
    oparamptr newops;
    stringpart *begpart, *endpart;
+   char *newkey;
 
    /* cannot form a parameter on a top-level page */
 
@@ -1589,6 +1640,10 @@ void makeparam(labelptr thislabel, char *key)
       return;
    }
 
+   /* check parameter against reserved PostScript names */
+
+   newkey = checkpostscriptname(key, NULL, NULL);
+
    /* First, place PARAM_START and PARAM_END structures at the	*/
    /* intended parameter boundaries				*/
 
@@ -1615,13 +1670,14 @@ void makeparam(labelptr thislabel, char *key)
       endpart = makesegment(&thislabel->string, NULL);
    }
    begpart->type = PARAM_START;
-   begpart->data.string = (char *)malloc(1 + strlen(key));
-   strcpy(begpart->data.string, key);
+   begpart->data.string = (char *)malloc(1 + strlen(newkey));
+   strcpy(begpart->data.string, newkey);
    endpart->type = PARAM_END;
+   endpart->data.string = (u_char *)NULL;
 
    /* Now move the sections of string to the object parameter */
 
-   newops = make_new_parameter(key);
+   newops = make_new_parameter(newkey);
    newops->next = topobject->params;
    topobject->params = newops;
    newops->type = XC_STRING;
@@ -1632,6 +1688,7 @@ void makeparam(labelptr thislabel, char *key)
 
    textend = 0;
    incr_changes(topobject);
+   free(newkey);
 }
 
 /*--------------------------------------------------------------*/
@@ -1668,7 +1725,7 @@ void searchinst(objectptr topobj, objectptr refobj, char *key)
    if (topobj == NULL) return;
 
    for (pgen = topobj->plist; pgen < topobj->plist + topobj->parts; pgen++) {
-      if ((*pgen)->type == OBJECT) {
+      if (IS_OBJINST(*pgen)) {
 	 tinst = TOOBJINST(pgen);
 	 destroyinst(tinst, refobj, key);
       }
@@ -1704,9 +1761,10 @@ void free_object_param(objectptr thisobj, oparamptr thisparam)
             searchinst(*(xobjs.userlibs[j].library + k), thisobj, key);
       }
    }
-   for (k = 0; k < xobjs.delbuffer.number; k++) {
-      searchinst(*(xobjs.delbuffer.library + k), thisobj, key);
-   }
+
+   /* Ensure that this parameter is not referred to in the undo records */
+   /* We could be kinder and gentler to the undo record here. . . */
+   flush_undo_stack();
 
    /* Also check through all instances on the library page */
    if (l >= 0)
@@ -1776,7 +1834,7 @@ void unmakeparam(labelptr thislabel, stringpart *thispart)
 
    for (pgen = topobject->plist; pgen < topobject->plist + topobject->parts;
 		pgen++) {
-      if ((*pgen)->type == LABEL) {
+      if (IS_LABEL(*pgen)) {
 	 plab = TOLABEL(pgen);
 	 for (strptr = plab->string; strptr != NULL; strptr = strptr->nextpart) {
 	    if (strptr->type == PARAM_START) {
@@ -1843,11 +1901,11 @@ void unparameterize(int mode)
 
    if (mode >= 0) {
       ptype = (short)param_select[mode];
-      if (!checkselect(ptype)) objectselect(ptype);
+      if (!checkselect(ptype)) select_element(ptype);
       if (!checkselect(ptype)) return;
    }
    else
-      ptype = SEL_ANY;
+      ptype = ALL_TYPES;
 
    if ((areastruct.selects == 1) && (mode == P_SUBSTRING) && textend > 0
 		&& textend < textpos) {
@@ -1903,11 +1961,11 @@ void parameterize(int mode, char *key)
 
    if (mode >= 0) {
       ptype = (short)param_select[mode];
-      if (!checkselect(ptype)) objectselect(ptype);
+      if (!checkselect(ptype)) select_element(ptype);
       if (!checkselect(ptype)) return;
    }
    else
-      ptype = SEL_ANY;
+      ptype = ALL_TYPES;
 
    for (fselect = areastruct.selectlist; fselect < areastruct.selectlist +
             areastruct.selects; fselect++) {
@@ -1923,7 +1981,7 @@ void parameterize(int mode, char *key)
       else
 	 makenumericalp(topobject->plist + (*fselect), mode, key);
    }
-   objectdeselect();
+   unselect_all();
    setparammarks(NULL);
 }
 
