@@ -234,9 +234,9 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
 	 pathdefaults(*newpath, 0, 0);
 	 newgen = (genericptr *)newpath;
 	 break;
-      case OBJECT:
+      case OBJINST:
          NEW_OBJINST(newobjinst, topobject);
-	 objectdefaults(*newobjinst, NULL, 0, 0);
+	 instancedefaults(*newobjinst, NULL, 0, 0);
 	 newgen = (genericptr *)newobjinst;
 	 break;
       default:
@@ -246,9 +246,7 @@ static PyObject *xc_new(PyObject *self, PyObject *args)
    } 
    topobject->parts++;
    incr_changes(topobject);
-#ifdef SCHEMA
    invalidate_netlist(topobject);
-#endif
 
    return PyInt_FromLong((long)(*newgen));
 }
@@ -274,7 +272,7 @@ char *type_to_string(int type)
 	 retstr = malloc(12);
 	 strcpy(retstr, "Bezier Curve");
 	 break;
-      case OBJECT:
+      case OBJINST:
 	 retstr = malloc(16);
 	 strcpy(retstr, "Object Instance");
 	 break;
@@ -307,7 +305,7 @@ int string_to_type(const char *etype)
    else if (!strcmp(etype, "Path"))
       return PATH;
    else if (!strcmp(etype, "Object Instance"))
-      return OBJECT;
+      return OBJINST;
    else
       return -1;
 }
@@ -454,6 +452,7 @@ stringpart *PySetStringParts(PyObject *dval)
 	 string = PyString_AsString(litem);
 	 if (!strcmp(string, "End Parameter")) {
 	    newpart->type = PARAM_END;
+	    newpart->data.string = (u_char *)NULL;
 	 }
 	 else if (!strcmp(string, "Tab Stop")) {
 	    newpart->type = TABSTOP;
@@ -883,7 +882,6 @@ static PyObject *xc_reset()
    return PyInt_FromLong(0L);
 }
 
-#ifdef SCHEMA
 /*--------------------------------------------------------------*/
 /* Get a netlist						*/
 /*--------------------------------------------------------------*/
@@ -907,7 +905,6 @@ static PyObject *xc_netlist()
    PyDict_SetItem(rdict, PyString_FromString("circuit"), pytoplevel(topobject));
    return rdict;
 }
-#endif
 
 /*--------------------------------------------------------------*/
 /* Load the specified file					*/
@@ -1091,7 +1088,7 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
    if (lstr != NULL)
       PyDict_SetItem(rdict, PyString_FromString("color"), lstr);
 
-   switch((*gelem)->type) {
+   switch(ELEMENTTYPE(*gelem)) {
       case LABEL:
 	 PyDict_SetItem(rdict, PyString_FromString("position"), 
 		make_pair(&(TOLABEL(gelem)->position)));
@@ -1101,10 +1098,8 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 		PyFloat_FromDouble((double)TOLABEL(gelem)->scale));
 	 PyDict_SetItem(rdict, PyString_FromString("justify"),
 		PyInt_FromLong((long)TOLABEL(gelem)->justify));
-#ifdef SCHEMA
 	 PyDict_SetItem(rdict, PyString_FromString("pin"),
 		PyInt_FromLong((long)TOLABEL(gelem)->pin));
-#endif
 	 lstr = PyGetStringParts(TOLABEL(gelem)->string);
 
 	 PyDict_SetItem(rdict, PyString_FromString("string"), lstr);
@@ -1159,7 +1154,7 @@ static PyObject *xc_getattr(PyObject *self, PyObject *args)
 	 }
 	 PyDict_SetItem(rdict, PyString_FromString("parts"), dlist);
 	 break;
-      case OBJECT:
+      case OBJINST:
 	 PyDict_SetItem(rdict, PyString_FromString("position"), 
 		make_pair(&(TOOBJINST(gelem)->position)));
 	 PyDict_SetItem(rdict, PyString_FromString("rotation"),
@@ -1201,7 +1196,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
    if ((dval = PyDict_GetItemString(attrdict, "type")) != NULL) {
       int dtype = string_to_type(PyString_AsString(dval));
       if (dtype < 0) return NULL;
-      if (dtype != (*gelem)->type) {
+      if (dtype != ELEMENTTYPE(*gelem)) {
          PyErr_SetString(PyExc_TypeError,
 		"Attempt to change the type of an object.");
          return NULL;
@@ -1219,7 +1214,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
       if (PyTuple_Check(dval) && PyTuple_Size(dval) == 2) {
          if ((pval = PyTuple_GetItem(dval, 0)) != NULL) {
             short xpos = (short)PyInt_AsLong(pval);
-            switch((*gelem)->type) {
+            switch(ELEMENTTYPE(*gelem)) {
 	       case ARC:
 	          TOARC(gelem)->position.x = xpos;
 	          calcarc(TOARC(gelem));
@@ -1227,7 +1222,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	       case LABEL:
 	          TOLABEL(gelem)->position.x = xpos;
 	          break;
-	       case OBJECT:
+	       case OBJINST:
 	          TOOBJINST(gelem)->position.x = xpos;
 	          break;
                default:
@@ -1238,7 +1233,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
          if ((pval = PyTuple_GetItem(dval, 1)) != NULL) {
             short ypos = (short)PyInt_AsLong(pval);
-            switch((*gelem)->type) {
+            switch(ELEMENTTYPE(*gelem)) {
 	       case ARC:
 	          TOARC(gelem)->position.y = ypos;
 	          calcarc(TOARC(gelem));
@@ -1246,7 +1241,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	       case LABEL:
 	          TOLABEL(gelem)->position.y = ypos;
 	          break;
-	       case OBJECT:
+	       case OBJINST:
 	          TOOBJINST(gelem)->position.y = ypos;
 	          break;
                default:
@@ -1263,7 +1258,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "style")) != NULL) {
       short dstyle = (short)PyInt_AsLong(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case POLYGON:
 	    TOPOLY(gelem)->style = dstyle;
 	    break;
@@ -1284,7 +1279,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "linewidth")) != NULL) {
       float dwidth = (float)PyFloat_AsDouble(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case POLYGON:
 	    TOPOLY(gelem)->width = dwidth;
 	    break;
@@ -1305,11 +1300,11 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "scale")) != NULL) {
       float dscale = (float)PyFloat_AsDouble(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case LABEL:
 	    TOLABEL(gelem)->scale = dscale;
 	    break;
-         case OBJECT:
+         case OBJINST:
 	    TOOBJINST(gelem)->scale = dscale;
 	    break;
          default:
@@ -1320,11 +1315,11 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    if ((dval = PyDict_GetItemString(attrdict, "rotation")) != NULL) {
       float drot = (float)PyFloat_AsDouble(dval);
-      switch((*gelem)->type) {
+      switch(ELEMENTTYPE(*gelem)) {
          case LABEL:
 	    TOLABEL(gelem)->rotation = drot;
 	    break;
-         case OBJECT:
+         case OBJINST:
 	    TOOBJINST(gelem)->rotation = drot;
 	    break;
          default:
@@ -1335,7 +1330,7 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 
    /* Dictionary entries specific to certain xcircuit types */
 
-   switch((*gelem)->type) {
+   switch(ELEMENTTYPE(*gelem)) {
       case LABEL:
 	 if ((dval = PyDict_GetItemString(attrdict, "justify")) != NULL) {
 	    TOLABEL(gelem)->justify = PyInt_AsLong(dval);
@@ -1344,11 +1339,9 @@ static PyObject *xc_setattr(PyObject *self, PyObject *args)
 	    freelabel(TOLABEL(gelem)->string);
 	    TOLABEL(gelem)->string = PySetStringParts(dval);
 	 }
-#ifdef SCHEMA
 	 if ((dval = PyDict_GetItemString(attrdict, "pin")) != NULL) {
 	    TOLABEL(gelem)->pin = PyInt_AsLong(dval);
 	 }
-#endif
 	 break;
       case ARC:
 	 if ((dval = PyDict_GetItemString(attrdict, "start angle")) != NULL) {
@@ -1519,12 +1512,10 @@ static PyObject *xc_set(PyObject *self, PyObject *args)
          xobjs.pagelist[cpage]->orient = 0;  /* Portrait */
    }
 
-#ifdef SCHEMA
    else if (!strcmp(sarg1, "xschema") || !strcmp(sarg1, "schema")) {
       areastruct.schemon = a;
       doxschema(OptionsEnableXSchemaButton, 1, NULL);
    }
-#endif
 #ifdef HAVE_XPM
    else if (!strcmp(sarg1, "toolbar")) {
       areastruct.toolbar_on = a;
@@ -1956,9 +1947,7 @@ static PyMethodDef xc_methods[] = {
 #ifdef HAVE_XPM
    {"newtool",		xc_newtool,	1},
 #endif
-#ifdef SCHEMA
    {"netlist",		xc_netlist,	1},
-#endif
    {NULL,	NULL}			/* sentinel */
 };
 
@@ -2118,7 +2107,6 @@ void loadrcfile()
       addnewcolorentry(xc_alloccolor("Brown"));
    }  
 
-#ifdef SCHEMA
    /* These colors must be enabled whether or not colors are overridden, */
    /* because they are needed by the schematic capture system.		 */
 
@@ -2126,7 +2114,6 @@ void loadrcfile()
    addnewcolorentry(xc_getlayoutcolor(GLOBALPINCOLOR));
    addnewcolorentry(xc_getlayoutcolor(INFOLABELCOLOR));
    addnewcolorentry(xc_getlayoutcolor(RATSNESTCOLOR));
-#endif
    addnewcolorentry(xc_getlayoutcolor(BBOXCOLOR));
 
    if (!(flags & KEYOVERRIDE))
