@@ -10,11 +10,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(DARWIN)
-#include <sys/malloc.h>
-#else
-#include <malloc.h>
-#endif
 #include <unistd.h>
 #include <time.h>
 #include <sys/wait.h>   /* for waitpid() */
@@ -64,6 +59,12 @@ extern Pixmap dbuf;
 #define vfork fork
 #endif
 
+int gs_state;		/* Track state of the renderer */
+
+#define GS_INIT 0	/* Initial state; gs is idle. */
+#define GS_PENDING 1	/* Drawing in progress; gs is busy. */
+#define GS_READY 2	/* Drawing done; gs is waiting for "next". */
+
 /*------------------------------------------------------*/
 /* Global variable definitions				*/
 /*------------------------------------------------------*/
@@ -82,14 +83,8 @@ Window mwin = 0;	/* "false" window hack to get gs	*/
 /* variable.							*/
 /*--------------------------------------------------------------*/
 
-void ghostinit()
+void ghostinit_local()
 {
-   gv = XInternAtom(dpy, "GHOSTVIEW", False);
-   gvc = XInternAtom(dpy, "GHOSTVIEW_COLORS", False);
-   gvpage = XInternAtom(dpy, "PAGE", False);
-   gvnext = XInternAtom(dpy, "NEXT", False);
-   gvdone = XInternAtom(dpy, "DONE", False);
-
    sprintf(_STR, "%ld %d %d %d %d %d %g %g %d %d %d %d",
 		0L, 0, 0, 0,
 		areastruct.width * 75 / 72,
@@ -101,6 +96,79 @@ void ghostinit()
    XChangeProperty(dpy, areastruct.areawin, gvc, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    XSync(dpy, False);
+
+   gs_state = GS_INIT;
+}
+
+/*------------------------------------------------------*/
+/* Client message handler.  Called by either the Tk	*/
+/* client message handler (see below) or the Xt handler	*/
+/* (see xcircuit.c).					*/
+/*							*/
+/* Returns True if a client message was received from	*/
+/* the ghostscript process, False if not.		*/
+/*------------------------------------------------------*/
+
+Boolean render_client(XEvent *eventPtr)
+{
+   if (eventPtr->xclient.message_type == gvpage) {
+#ifdef GS_DEBUG
+      fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
+#endif
+      mwin = eventPtr->xclient.data.l[0];
+      Wprintf("Background finished.");
+      XDefineCursor(dpy, areastruct.areawin, CROSS);
+
+      /* Mark this as the most recently rendered background, so we don't	*/
+      /* have to render more than necessary.					*/
+      areastruct.lastbackground = xobjs.pagelist[areastruct.page]->background.name;
+      gs_state = GS_READY;
+      drawarea(areastruct.area, NULL, NULL);
+   }
+   else if (eventPtr->xclient.message_type == gvdone) {
+#ifdef GS_DEBUG
+      fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
+#endif
+      mwin = 0;
+      gs_state = GS_INIT;
+   } 
+   else {
+      return False;
+   }
+   return True;
+}
+
+/*------------------------------------------------------*/
+/* Tk client handler procedure for the above routine.	*/
+/*------------------------------------------------------*/
+
+#ifdef TCL_WRAPPER
+
+void handle_client(ClientData clientData, XEvent *eventPtr)
+{
+   if (render_client(eventPtr) == False)
+      fprintf(stderr, "Xcircuit: Received unknown client message\n");
+}
+
+#endif
+
+/*------------------------------------------------------*/
+/* Global initialization				*/
+/*------------------------------------------------------*/
+
+void ghostinit()
+{
+   gv = XInternAtom(dpy, "GHOSTVIEW", False);
+   gvc = XInternAtom(dpy, "GHOSTVIEW_COLORS", False);
+   gvpage = XInternAtom(dpy, "PAGE", False);
+   gvnext = XInternAtom(dpy, "NEXT", False);
+   gvdone = XInternAtom(dpy, "DONE", False);
+
+   ghostinit_local();
+
+#ifdef TCL_WRAPPER
+   Tk_CreateClientMessageHandler((Tk_ClientMessageProc *)handle_client);
+#endif
 }
 
 /*------------------------------------------------------*/
@@ -131,77 +199,21 @@ void send_client(Atom msg)
 
 void ask_for_next()
 {
+   if (gs_state != GS_READY) {
+      /* If we're in the middle of rendering something, halt	*/
+      /* immediately and start over.				*/
+      if (gs_state == GS_PENDING)
+	 reset_gs();
+      return;
+   }
    XSync(dpy, False);
+   gs_state = GS_PENDING;
    send_client(gvnext);
 #ifdef GS_DEBUG
    fprintf(stdout, "Xcircuit: Sent NEXT message to ghostscript\n");
 #endif
 }
 
-/*------------------------------------------------------*/
-/* Wait for the gs process to return "PAGE"		*/
-/* (Block until gs is finished)				*/
-/*------------------------------------------------------*/
-
-void wait_for_page()
-{
-   XEvent event;
-
-   for(;;) {
-      XNextEvent(dpy, &event);
-      if (event.type == ClientMessage) {
-	 if (event.xclient.message_type == gvpage) break;
-         else if (event.xclient.message_type == gvdone) {
-#ifdef GS_DEBUG
-	    fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
-#endif
-	    /* Catastrophic error in ghostscript */
-	    return;
-         }
-      } 
-      else if (event.type == KeyPress) {	/* Check for Ctrl-C break */
-	 XKeyEvent *keyevent = (XKeyEvent *)(&event);
-	 if (keyevent->state == 4 && keyevent->keycode == 54) {
-	    Wprintf("Ctrl-C break from render loop");
-	    break;
-	 }
-	 else {
-	    Fprintf(stderr, "state = %d, keycode = %d\n", keyevent->state,
-		keyevent->keycode);
-	    xcDispatchEvent(&event);
-	 }
-      }
-      else xcDispatchEvent(&event);	
-   }
-#ifdef GS_DEBUG
-   fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
-#endif
-   mwin = event.xclient.data.l[0];
-}
-
-/*------------------------------------------------------*/
-
-void wait_for_done()
-{
-   XEvent event;
-
-   for(;;) {
-      XNextEvent(dpy, &event);
-      if (event.type == ClientMessage) {
-	 if (event.xclient.message_type == gvdone) break;
-#ifdef GS_DEBUG
-         else if (event.xclient.message_type == gvpage)
-	    fprintf(stdout, "Xcircuit: Received PAGE message from ghostscript\n");
-#endif
-      }
-      else xcDispatchEvent(&event);
-   }
-#ifdef GS_DEBUG
-   fprintf(stdout, "Xcircuit: Received DONE message from ghostscript\n");
-#endif
-   mwin = 0;
-}
-
 /*--------------------------------------------------------*/
 /* Start a ghostscript process and arrange the I/O pipes  */
 /* (Commented lines cause ghostscript to relay its stderr */
@@ -293,7 +305,7 @@ void start_gs()
 	 Flush(stderr);
          execlp(GS_EXEC, "gs", "-dNOPAUSE", "-", NULL);
 	 gsproc = -1;
-	 Fprintf(stderr, "Exec of gs failed\n");
+	 fprintf(stderr, "Exec of gs failed\n");
 	 return;
       }
       else if (gsproc < 0) {
@@ -365,7 +377,7 @@ void bg_get_bbox()
 
    fname = xobjs.pagelist[areastruct.page]->background.name;
    if ((fi = fopen(fname, "r")) == NULL) {
-      Fprintf(stderr, "Failure to open background file to get bounding box info\n");
+      fprintf(stderr, "Failure to open background file to get bounding box info\n");
       return;
    }
    parse_bg(fi, (FILE *)NULL);
@@ -416,10 +428,10 @@ void readbackground(FILE *fi)
    sprintf(file_in, "@%s/XXXXXX", xobjs.tempdir);
 
    tfd = mkstemp(file_in + 1);
-   if (tfd == -1) Fprintf(stderr, "Error generating temporary filename\n");
+   if (tfd == -1) fprintf(stderr, "Error generating temporary filename\n");
    else {
       if ((fbg = fdopen(tfd, "w")) == NULL) {
-	 Fprintf(stderr, "Error opening temporary file \"%s\"\n", file_in + 1);
+	 fprintf(stderr, "Error opening temporary file \"%s\"\n", file_in + 1);
       }
    }
 
@@ -430,7 +442,7 @@ void readbackground(FILE *fi)
 
    if (fbg != (FILE *)NULL) {
       fclose(fbg);
-      call_loadbg(file_in);
+      register_bg(file_in);
    }
    free(file_in);
 }
@@ -449,7 +461,7 @@ void savebackground(FILE *fo, char *psfilename)
    if (fname[0] == '@') fname++;
 
    if ((psf = fopen(fname, "r")) == NULL) {
-      Fprintf(stderr, "Error opening background file \"%s\" for reading.\n", fname);
+      fprintf(stderr, "Error opening background file \"%s\" for reading.\n", fname);
       return;
    }
 
@@ -462,12 +474,14 @@ void savebackground(FILE *fo, char *psfilename)
    fclose(psf);
 }
 
-/*------------------------------------------------------*/
-/* Load a generic (non-xcircuit) postscript file as the */
-/* background for the page.				*/
-/*------------------------------------------------------*/
+/*--------------------------------------------------------------*/
+/* Set up a page to render a PostScript image when redrawing.	*/
+/* This includes starting the ghostscript process if it has	*/
+/* not already been started.  This routine does not draw the	*/
+/* image, which is done on refresh.				*/
+/*--------------------------------------------------------------*/
 
-void call_loadbg(char *gsfile)
+void register_bg(char *gsfile)
 {
    if (gsproc < 0)
       start_gs();
@@ -479,14 +493,20 @@ void call_loadbg(char *gsfile)
    strcpy(xobjs.pagelist[areastruct.page]->background.name, gsfile);
 }
 
-/*--------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Load a generic (non-xcircuit) postscript file as the */
+/* background for the page.   This function is called	*/
+/* by the file import routine, and so it completes by	*/
+/* running zoomview(), which redraws the image and	*/
+/* the page.						*/
+/*------------------------------------------------------*/
 
 void loadbackground()
 {
-   call_loadbg(_STR2);
+   register_bg(_STR2);
    bg_get_bbox();
    updatepagebounds(topobject);
-   zoomview(NULL, NULL, NULL);
+   zoomview(areastruct.area, NULL, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -502,7 +522,8 @@ void send_to_gs(char *text)
 }
 
 /*------------------------------------------------------*/
-/* Call ghostscript to render the background		*/
+/* Call ghostscript to render the background into the	*/
+/* pixmap buffer.					*/
 /*------------------------------------------------------*/
 
 int renderbackground()
@@ -527,25 +548,30 @@ int renderbackground()
 
    if (xobjs.pagelist[areastruct.page]->background.name == (char *)NULL)
       return -1;
-   else if (areastruct.lastbackground == xobjs.pagelist[areastruct.page]->background.name)
+   else if (areastruct.lastbackground
+		== xobjs.pagelist[areastruct.page]->background.name) {
       return 0;
+   }
 
    if (is_page(topobject) == -1)
       return -1;
 
-   ask_for_next();
-
    bgfile = xobjs.pagelist[areastruct.page]->background.name;
    if (*bgfile == '@') bgfile++;
 
-#ifdef GS_DEBUG
-   fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
-#endif
+   /* Ask ghostscript to produce the next page */
+   ask_for_next();
+
+   /* Set the last background name to NULL; this will get the 	*/
+   /* value when the rendering is done.				*/
+
+   areastruct.lastbackground = NULL;
 
    /* write scale and position to ghostscript 		*/
    /* and tell ghostscript to run the requested file	*/
 
-/*   send_to_gs("/GSobj save def\n"); */
+   send_to_gs("/GSobj save def\n");
+   send_to_gs("/setpagedevice {pop} def\n");
    send_to_gs("gsave\n");
    sprintf(_STR, "%3.2f %3.2f translate\n", psxpos, psypos);
    send_to_gs(_STR);
@@ -553,31 +579,32 @@ int renderbackground()
    send_to_gs(_STR);
    sprintf(_STR, "(%s) run\n", bgfile);
    send_to_gs(_STR);
-/* send_to_gs("GSobj restore\n"); */
+   send_to_gs("GSobj restore\n");
    send_to_gs("grestore\n");
 
-   /* ask ghostscript to produce the next page */
-
-   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
+#ifdef GS_DEBUG
+   fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
+#endif
    Wprintf("Rendering background image.");
-   wait_for_page();
-   Wprintf("Background finished.");
-   XDefineCursor(dpy, areastruct.areawin, CROSS);
+   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
 
-   /* Mark this as the most recently rendered background, so we don't	*/
-   /* have to render more than necessary.				*/
-   areastruct.lastbackground = xobjs.pagelist[areastruct.page]->background.name;
+   /* The page will be refreshed when we receive confirmation	*/
+   /* from ghostscript that the buffer has been rendered.	*/
 
    return 0;
 }
 
-/*----------------------------------------------------------*/
-/* Copy the rendered background pixmap to the buffer pixmap */
-/*----------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Copy the rendered background pixmap to the window.	*/
+/*------------------------------------------------------*/
 
 int copybackground()
 {
-   /* only draw on a top-level page */
+   /* Don't copy if the buffer is not ready to use */
+   if (gs_state != GS_READY)
+      return -1;
+
+   /* Only draw on a top-level page */
    if (is_page(topobject) == -1)
       return -1;
 
@@ -593,18 +620,17 @@ int copybackground()
 }
 
 /*------------------------------------------------------*/
-/* Exit ghostscript. . . gently				*/
+/* Exit ghostscript. . . not so gently			*/
 /*------------------------------------------------------*/
 
 int exit_gs()
 {
    if (gsproc < 0) return -1;	/* gs not running */
 
-   send_to_gs("quit\n");
-   ask_for_next();
 #ifdef GS_DEBUG
    fprintf(stdout, "Waiting for gs to exit\n");
 #endif
+   kill(gsproc, SIGKILL);
    waitpid(gsproc, NULL, 0);      
 #ifdef GS_DEBUG
    fprintf(stdout, "gs has exited\n");
@@ -612,6 +638,7 @@ int exit_gs()
 
    mwin = 0;
    gsproc = -1;
+   gs_state = GS_INIT;
 
    return 0;
 }
@@ -625,7 +652,7 @@ int reset_gs()
    if (gsproc < 0) return -1;
 
    exit_gs();
-   ghostinit();
+   ghostinit_local();
    start_gs();
 
    return 0;
