@@ -228,21 +228,22 @@ void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 /*----------------------------------------------------------------------*/
 /* connectivity():  Find electrical connections into a node.		*/
 /* (does recursive search on object heirarchy)				*/
-/* Returns net number found (0 if no network found, -1 if error)	*/
 /*----------------------------------------------------------------------*/
 
-int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
+void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
    short *gsel;
    genericptr ggen;
-   int depth, netid = -1;
+   Genericlist *netlist = NULL;
+   int depth, lbus, netid, subnetid;
+   buslist *sbus;
    pushlistptr seltop, nextptr;
    objectptr nettop, thisobj;
    char *snew;
    stringpart *ppin;
 
    /* erase any existing highlighted network */
-   highlightnet(topobject, areastruct.topinstance, -1, 0);
+   highlightnetlist(topobject, areastruct.topinstance, 0);
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
    seltop->thisinst = areastruct.topinstance;
@@ -270,45 +271,60 @@ int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    else
       ggen = recurselect(POLYGON | LABEL, &seltop);
 
-   /* Determine the netid and the topmost object in which that	*/
+   /* Determine the net and the topmost object in which that	*/
    /* net appears.  Then build the transformation matrix down	*/
    /* to that object.  highlightnet() should end by popping the */
    /* entire matrix stack.					*/
 
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
-         destroynets();
+         destroynets(topobject);
          createnets(areastruct.topinstance);
       }
-      if ((netid = is_resolved(&ggen, seltop, &nettop)) != 0) {
-         /* Fprintf(stdout, "Net ID is %d in object %s\n", netid, nettop->name); */
+      if ((netlist = is_resolved(&ggen, seltop, &nettop)) != NULL) {
          depth = pushnetwork(seltop, nettop);
          /* Fprintf(stdout, ">> Pushed network %d levels deep\n", depth); */
          nextptr = seltop;
          while (nextptr->thisinst->thisobject != nettop)
 	    nextptr = nextptr->next;
 
-	 nextptr->thisinst->thisobject->highlight.netid = netid;
+	 nextptr->thisinst->thisobject->highlight.netlist = netlist;
 	 nextptr->thisinst->thisobject->highlight.thisinst = nextptr->thisinst;
+	 highlightnetlist(nettop, nextptr->thisinst, 1);
 	    
-         highlightnet(nettop, nextptr->thisinst, netid, 1);
-
          /* pop the matrix stack */
          while (depth-- > 0) 
 	    UPopCTM();
 
 	 /* print the net name to the message window */
 
-	 ppin = nettopin(netid, nettop, NULL);
-         snew = textprint(ppin, areastruct.topinstance);
-
-	 sprintf(_STR, "Network is \"%s\" in %s", snew, nettop->name);
-	 Wprintf(_STR);
+	 if (netlist->subnets == 0) {
+	    ppin = nettopin(netlist->net.id, nettop, NULL);
+            snew = textprint(ppin, areastruct.topinstance);
+	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
+            free(snew);
+	 }
+	 else {
+	    char *sptr;
+	    sprintf(_STR2, "Network(s): ");
+	    sptr = _STR2 + strlen(_STR2);
+	    for (lbus = 0; lbus < netlist->subnets; lbus++) {
+	       sbus = netlist->net.list + lbus;
+	       netid = sbus->netid;
+	       subnetid = sbus->subnetid;
+	       ppin = nettopin(netid, nettop, NULL);
+               snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
+	       sprintf(sptr, "%s ", snew);
+	       sptr += strlen(snew) + 1;
+	       free(snew);
+	    }
+	    sprintf(sptr, "in %s", nettop->name);
+	 }
+	 Wprintf(_STR2);
 
 #ifdef TCL_WRAPPER
 	 Tcl_SetObjResult(xcinterp, Tcl_NewStringObj(snew, strlen(snew)));
 #endif
-         free(snew);
       }
       else
 	 Wprintf("Selected element is not part of a valid network.");
@@ -325,7 +341,6 @@ int connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
       free(seltop);
       seltop = nextptr;
    }
-   return netid;
 }
 
 /*--------------------------------------------------------------*/
@@ -348,7 +363,7 @@ Boolean setobjecttype(objectptr cschem)
 	 cschem->schemtype = SYMBOL;
       if (cschem->symschem == NULL) {
          for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
-            if ((*cgen)->type == LABEL) {
+            if (IS_LABEL(*cgen)) {
                clab = TOLABEL(cgen);
 	       if (clab->pin == INFO) {
 	          cschem->schemtype = FUNDAMENTAL;
@@ -431,7 +446,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
       }
 
    if (savetype >= 0) {
-      objectdeselect();
+      unselect_all();
       drawarea(NULL, NULL, NULL);
       Wprintf(typestr);
    }
@@ -569,7 +584,7 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
    if (level == HIERARCHY_LIMIT) return -1;	/* sanity check */
 
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
-      if ((*cgen)->type == OBJECT) {
+      if (IS_OBJINST(*cgen)) {
 	 objectptr cobj = TOOBJINST(cgen)->thisobject;
 
 	 if (cobj->symschem != NULL) {
@@ -655,7 +670,7 @@ void copypinlabel(labelptr pinlab)
 
       for (tgen = schemobj->plist; tgen < schemobj->plist + schemobj->parts;
 		tgen++) {
-         if ((*tgen)->type == LABEL) {
+         if (IS_LABEL(*tgen)) {
 	    tlabel = TOLABEL(tgen);
 	    if (!stringcomp(tlabel->string, pinlab->string)) return;
          }
@@ -757,7 +772,7 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
    if (other == NULL) return;
 
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
-      if ((*tgen)->type == LABEL) {
+      if (IS_LABEL(*tgen)) {
 	 tlab = TOLABEL(tgen);
 	 if (!stringcomp(tlab->string, oldstring)) {
 	    if (newlabel != NULL) {
@@ -788,7 +803,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
    if (areastruct.schemon == False) return;
 
    if (eventmode == PRESS_MODE || eventmode == COPY2_MODE)
-      objectdelete(NORMAL);
+      standard_element_delete(NORMAL);
 
    /* Create symbol or schematic, if allowed by mode */
 
@@ -853,7 +868,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 
       for (plab = savepage->plist; plab < savepage->plist + savepage->parts;
 		plab++) {
-	 if ((*plab)->type == LABEL) {
+	 if (IS_LABEL(*plab)) {
 	    genericptr *tgen;
 	    labelptr tlab, lpin = (labelptr)*plab;
 
@@ -864,7 +879,7 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 	       lflag = False;
                for (tgen = topobject->plist; tgen <
 			topobject->plist + topobject->parts; tgen++) {
-		  if ((*tgen)->type == LABEL) {
+		  if (IS_LABEL(*tgen)) {
 		     tlab = TOLABEL(tgen);
 	             if (!stringcomp(tlab->string, lpin->string)) lflag = True;
 		  }
