@@ -75,7 +75,7 @@ Boolean checkselect(short value)
  
    value &= areastruct.filter;	/* apply the selection filter */
 
-   if (areastruct.selects == 0) objectselect(value);
+   if (areastruct.selects == 0) select_element(value);
    if (areastruct.selects == 0) return False;
    for (check = areastruct.selectlist; check < areastruct.selectlist +
 	areastruct.selects; check++)
@@ -105,7 +105,7 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
 {
    genericptr elementptr = *(curobj->plist + instance);
 
-   switch ((*(curobj->plist + instance))->type) {
+   switch (ELEMENTTYPE(*(curobj->plist + instance))) {
       case ARC:
          UDrawArc((arcptr)elementptr);
 	 break;
@@ -121,7 +121,7 @@ void geneasydraw(short instance, short mode, objectptr curobj, objinstptr curins
       case LABEL:
          UDrawString((labelptr)elementptr, mode, curinst);
 	 break;
-      case OBJECT:
+      case OBJINST:
          UDrawObject((objinstptr)elementptr, SINGLE, mode, NULL);
          break;
    }
@@ -142,7 +142,6 @@ void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
    XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
-/* XSetForeground(dpy, areastruct.gc, PARAMCOLOR ^ BACKGROUND); */
    XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
 }
@@ -336,7 +335,7 @@ Boolean pathselect(genericptr *curgen, short class)
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
 
-short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
+short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr selinst)
 {
    short	*newselect, *foundtmp, *foundlist;
 
@@ -355,7 +354,7 @@ short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
       unselect = True;
       class = -class;
    }
-   class &= areastruct.filter;	/* apply the selection filter */
+   class &= areastruct.filter;		/* apply the selection filter */
 
    /* Loop through all elements found underneath the cursor */
 
@@ -371,7 +370,7 @@ short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
 
       /* check for position inside the bounding box of one of the objects */
 
-      if ((*curgen)->type == (class & OBJECT)) {
+      if ((*curgen)->type == (class & OBJINST)) {
 
 	 objinstbbox(TOOBJINST(curgen), newboxpts);
 
@@ -562,6 +561,11 @@ short *genobjectselect(short class, objectptr selobj, objinstptr selinst)
    /* snap the coordinates */
 
    u2u_snap(&areastruct.save);
+
+   /* Register the selection as an undo event */
+   register_for_undo(XCF_Select, mode, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+
    return newselect;
 }
 
@@ -574,7 +578,7 @@ Boolean areaelement(genericptr *curgen)
    Boolean selected;
    pointlist    currentpt;
 
-   switch((*curgen)->type) {
+   switch(ELEMENTTYPE(*curgen)) {
 
       case(ARC):
 	   /* check center of arcs */
@@ -650,8 +654,8 @@ void selectarea()
       /* apply the selection filter */
       if (!((*curgen)->type & areastruct.filter)) continue;
 
-      switch((*curgen)->type) {
-	case(OBJECT):
+      switch(ELEMENTTYPE(*curgen)) {
+	case(OBJINST):
            /* check for instance of object inside area box */
            selected = (TOOBJINST(curgen)->position.x < areastruct.save.x) &&
 		(TOOBJINST(curgen)->position.x > areastruct.origin.x) &&
@@ -768,6 +772,10 @@ void selectarea()
       textpos = textend = 0;
    }
 
+   /* Register the selection as an undo event */
+   register_for_undo(XCF_Select, UNDO_DONE, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+
    /* Drawing of selected objects will take place when drawarea() is */
    /* executed after the button release.			     */
 }
@@ -782,7 +790,7 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
       if (areastruct.selects == 0)
 	 Wprintf("Nothing to deselect!");
       else if (areastruct.selects == 1)
-	 objectdeselect();
+	 unselect_all();
       else {
          eventmode = DESEL_MODE;
          Wprintf("Click on element to deselect");
@@ -790,11 +798,23 @@ void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
    }
 }
 
+/*------------------------------------------------------*/
+/* Redraw all the selected objects in the select color.	*/
+/*------------------------------------------------------*/
+
+void draw_all_selected()
+{
+   int j;
+
+   for (j = 0; j < areastruct.selects; j++)
+      gendrawselected(areastruct.selectlist + j, topobject, areastruct.topinstance);
+}
+
 /*---------------------------------------------------------*/
 /* Redraw all the selected objects in their normal colors. */
 /*---------------------------------------------------------*/
 
-void drawselects(objectptr thisobj, objinstptr thisinst)
+void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 {
    short *lastselect;
 
@@ -832,7 +852,7 @@ void freeselects()
 /* items back to default values.				*/
 /*--------------------------------------------------------------*/
 
-void clearselects()
+void clearselects_noundo()
 {
    if (areastruct.selects > 0) {
       freeselects();
@@ -842,14 +862,27 @@ void clearselects()
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Same as above, but registers an undo event.			*/
+/*--------------------------------------------------------------*/
+
+void clearselects()
+{
+   if (areastruct.selects > 0) {
+      register_for_undo(XCF_Select, UNDO_DONE, areastruct.topinstance,
+		NULL, 0);
+      clearselects_noundo();
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Unselect all the selected objects and free memory from the	*/
 /* selection list.						*/
 /*--------------------------------------------------------------*/
 
-void objectdeselect()
+void unselect_all()
 {
-   drawselects(topobject, areastruct.topinstance);
+   draw_normal_selected(topobject, areastruct.topinstance);
    clearselects();
 }
 
@@ -878,18 +911,18 @@ genericptr recurselect(short class, pushlistptr *seltop)
    selinst = (*seltop)->thisinst;
    selobj = selinst->thisobject;
 
-   if ((selectobj = genobjectselect(class, selobj, selinst)) != NULL) {
+   if ((selectobj = genselectelement(class, UNDO_DONE, selobj, selinst)) != NULL) {
       rgen = *(selobj->plist + (*selectobj));
-      /* Fprintf(stdout, "%s found in object %s\n", (rgen->type == LABEL) ?
+      /* Fprintf(stdout, "%s found in object %s\n", (IS_LABEL(rgen)) ?
 			"label" : "polygon", selobj->name); */
-      drawselects(selobj, selinst);
+      draw_normal_selected(selobj, selinst);
       clearselects();
    }
-   else if ((selectobj = genobjectselect(OBJECT, selobj, selinst)) != NULL) {
+   else if ((selectobj = genselectelement(OBJINST, UNDO_DONE, selobj, selinst)) != NULL) {
       pushlistptr selnew = (pushlistptr)malloc(sizeof(pushlist));
 
       selinst = TOOBJINST(selobj->plist + (*selectobj));	
-      drawselects(selobj, selinst);
+      draw_normal_selected(selobj, selinst);
       clearselects();
 
       /* Link hierarchy information to the pushlist linked list */
