@@ -5,7 +5,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>   /* for isprint() */
+#include <ctype.h>   /* for isprint() and isdigit() */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -208,9 +208,17 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
 
    if (strptr->type == PARAM_START)
       nextptr = linkstring(thisinst, strptr);
-   else if (strptr->type == PARAM_END)
+   else if (strptr->type == PARAM_END) {
       strptr->nextpart = NULL;
 
+      /* Parameters that have a non-NULL entry in data have	*/
+      /* been promoted from an expression or numerical value	*/
+      /* to a string.  The memory allocated for this string	*/
+      /* should be free'd.					*/
+
+      if (strptr->data.string != (u_char *)NULL)
+	 freelabel((stringpart *)(strptr->data.string));
+   }
    return nextptr;
 }
 
@@ -221,7 +229,7 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
 
 stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisinst)
 {
-   stringpart *strptr, *nextptr;
+   stringpart *strptr = NULL, *nextptr;
    char *key;
    oparamptr ops;
 
@@ -248,7 +256,14 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
 	    Fprintf(stderr, "Error in deletestring:  Bad parameter %s found\n", key); 
 	 }
 	 else {
-	    ops->parameter.string = dstr->nextpart;
+	    switch(ops->type) {
+	       case XC_STRING:
+		  ops->parameter.string = dstr->nextpart;
+		  break;
+	       default:
+		  /* What to be done here? */
+		  break;
+	    }
 	 }
       }
       /* If this is the end of a parameter, we have to link the		*/
@@ -268,7 +283,8 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
    free(dstr);
 
    /* attempt to merge, if legal */
-   mergestring(strptr);
+   if (strptr)
+      mergestring(strptr);
 
    return strptr;
 }
@@ -307,7 +323,7 @@ stringpart *mergestring(stringpart *firststr)
 stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 {
    char *key;
-   stringpart *nextptr = NULL, *tmpptr;
+   stringpart *tmpptr, *nextptr = NULL;
    static stringpart *promote = NULL;
    oparamptr ops;
 
@@ -334,22 +350,28 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 
    if (ops->type != XC_STRING) {
 
-      /* Generate static string for promoting numerical parameters */
       if (promote == NULL) {
-	 tmpptr = makesegment(&promote, NULL);
-	 tmpptr->type = TEXT_STRING;
-	 tmpptr->data.string = (char *)malloc(13);
-	 tmpptr = makesegment(&promote, NULL);
-	 tmpptr->type = PARAM_END;
+         /* Generate static string for promoting numerical parameters */
+         tmpptr = makesegment(&promote, NULL);
+         tmpptr->type = TEXT_STRING;
+         tmpptr = makesegment(&promote, NULL);
+         tmpptr->type = PARAM_END;
       }
+      else
+	 free(promote->data.string);
 
       /* Promote numerical type to string */
-      if (ops->type == XC_INT)
+      if (ops->type == XC_INT) {
+	 promote->data.string = (char *)malloc(13);
 	 sprintf(promote->data.string, "%12d", ops->parameter.ivalue);
-      else if (ops->type == XC_FLOAT)
+      }
+      else if (ops->type == XC_FLOAT) {
+	 promote->data.string = (char *)malloc(13);
 	 sprintf(promote->data.string, "%g", (double)(ops->parameter.fvalue));
+      }
       else
 	 promote->data.string = evaluate_expr(ops);
+
       nextptr = promote;
    }
    else
@@ -523,6 +545,83 @@ char *textprint(stringpart *strtop, objinstptr localinst)
    return xcstringtostring(strtop, localinst, True);
 }
 
+/*----------------------------------------------------------------------*/
+/* textprintsubnet() is like textprint(), except that strings in bus	*/
+/* notation are reduced to just the single subnet.			*/
+/*----------------------------------------------------------------------*/
+
+char *textprintsubnet(stringpart *strtop, objinstptr localinst, int subnet)
+{
+   char *newstr, *busptr, *endptr, *substr;
+
+   newstr = xcstringtostring(strtop, localinst, True);
+   if (subnet >= 0) {
+      busptr = strchr(newstr, areastruct.buschar);
+      if (busptr != NULL) {
+	 endptr = find_delimiter(busptr);
+	 if (endptr != NULL) {
+	    if (busptr == newstr)
+	       sprintf(newstr, "%d", subnet);
+	    else {
+	       substr = strdup(newstr);
+	       busptr++;
+	       sprintf(substr + (int)(busptr - newstr), "%d%s", subnet, endptr);
+	       free(newstr);
+	       return substr;
+	    }
+	 }
+      }
+      else {
+	 /* Promote a non-bus label to a bus label */
+	 substr = malloc(10 + strlen(newstr));
+	 strcpy(substr, newstr);
+	 endptr = substr;
+	 while ((*endptr) != '\0') endptr++;
+	 sprintf(endptr, "%c%d%c", areastruct.buschar, subnet,
+		standard_delimiter_end(areastruct.buschar));
+	 free(newstr);
+	 return substr;
+      }
+   }
+   return newstr;
+}
+
+/*----------------------------------------------------------------------*/
+/* Another variant:  Print a subnet list according to the entries in	*/
+/* a netlist (Genericlist *) pointer.  This includes the condition in	*/
+/* which the list is a single wire, not a bus.  If "pinstring" is non-	*/
+/* NULL, it will be used as the name of the subnet.  Otherwise, 	*/
+/* "prefix" will be used, and will be appended with the net ID of the	*/
+/* first net in the sublist to make the identifier unique.		*/
+/*----------------------------------------------------------------------*/
+
+char *textprintnet(char *prefix, char *pinstring, Genericlist *sublist)
+{
+   char *newstr, *sptr;
+   buslist *sbus;
+   int i;
+
+   if (sublist->subnets == 0) {
+      newstr = (char *)malloc(strlen(prefix) + 10); 
+      sprintf(newstr, "%s%d", prefix, sublist->net.id);
+   }
+   else {	/* just make a comma-separated list */
+      newstr = (char *)malloc(strlen(prefix) + 20 + 3 * sublist->subnets);
+      sbus = sublist->net.list;
+      sprintf(newstr, "%s%d%c", prefix, sbus->netid, areastruct.buschar);
+      for (i = 0; i < sublist->subnets; i++) {
+	 sbus = sublist->net.list + i;
+	 sptr = newstr + strlen(newstr);
+	 if (i != 0)
+	    strcat(sptr++, ",");
+	 sprintf(sptr, "%d", sbus->subnetid);
+      }
+      sptr = newstr + strlen(newstr);
+      sprintf(sptr, "%c", standard_delimiter_end(areastruct.buschar));
+   }
+   return newstr;
+}
+
 /*----------------------------------------------------------------------*/
 /* Test equivalence of the text string parts of a label with the	*/
 /* indicated char * string.						*/
@@ -635,18 +734,208 @@ Boolean isisolatin1(int fontnumber)
    return False;
 }
 
+/*----------------------------------------------------------------------*/
+/* For a label representing a single bus subnet, return the index of	*/
+/* the subnet.								*/
+/*----------------------------------------------------------------------*/
+
+int sub_bus_idx(labelptr thislab, objinstptr thisinst)
+{
+   stringpart *strptr;
+   char *busptr;
+   int busidx;
+
+   for (strptr = thislab->string; strptr != NULL; strptr =
+		nextstringpart(strptr, thisinst)) {
+      if (strptr->type == TEXT_STRING) {
+	 if ((busptr = strchr(strptr->data.string, areastruct.buschar)) != NULL) {
+	    if (sscanf(++busptr, "%d", &busidx) == 1)
+	       return busidx;
+	 }
+	 if (sscanf(strptr->data.string, "%d", &busidx) == 1)
+	    return busidx;
+      }
+   }
+   return -1;
+}
+
+/*----------------------------------------------------------------------*/
+/* The following routine is like sub_bus_idx but returns TRUE or FALSE	*/
+/* depending on whether the label was determined to be in bus notation	*/
+/* or not.  Note that sub_bux_idx may be run on sub-bus names (those	*/
+/* that are internally generated from labels in bus notation), but	*/
+/* pin_is_bus should not, because pin numbers in bus notation get the	*/
+/* bus delimiters stripped from them.					*/
+/*----------------------------------------------------------------------*/
+
+Boolean pin_is_bus(labelptr thislab, objinstptr thisinst)
+{
+   stringpart *strptr;
+   char *busptr;
+   int busidx;
+   Boolean found_delimiter = FALSE;
+
+   for (strptr = thislab->string; strptr != NULL; strptr =
+		nextstringpart(strptr, thisinst)) {
+      if (strptr->type == TEXT_STRING) {
+	 if ((busptr = strchr(strptr->data.string, areastruct.buschar)) != NULL) {
+	    if (isdigit(*(++busptr)))
+	       return TRUE;
+	    else
+	       found_delimiter = TRUE;
+	 }
+	 else if (found_delimiter == TRUE) {
+	    return (isdigit(*(strptr->data.string))) ? TRUE : FALSE;
+	 }
+      }
+   }
+   return FALSE;
+}
+
+/*----------------------------------------------------------------------*/
+/* When encountering a label with bus notation, create a list of	*/
+/* subnets belonging to the bus.  Return the list as a pointer to a	*/
+/* Genericlist structure.  This structure is statically allocated and	*/
+/* is expected to have its contents copied into the target netlist	*/
+/* element.								*/
+/*									*/
+/* Unlike the above routine, this routine prints the original string	*/
+/* into a char* array using textprint() so that escape sequences are	*/
+/* removed and will not affect the result.				*/
+/*									*/
+/* To speed things up, the calling routine should have already called	*/
+/* pin_is_bus() to determine if the label does indeed represent a bus.	*/
+/* break_up_bus() is much slower in determining this.  If break_up_bus	*/
+/* is passed a string that cannot be identified as a bus, then it	*/
+/* returns a NULL pointer.						*/
+/*									*/
+/* If netlist points to a structure with no subnets, then its net ID	*/
+/* is the starting point for the nets returned by break_up_bus.  	*/
+/* Otherwise, netlist is assumed to be a valid netlist for a bus that	*/
+/* matches "blab", and we will use net IDs from this list.		*/
+/*----------------------------------------------------------------------*/
+
+Genericlist *break_up_bus(labelptr blab, objinstptr thisinst, Genericlist *netlist)
+{
+   static Genericlist *subnets = NULL;
+   char *tptr;
+   buslist *sbus, *jbus;
+   int bpos, istart, iend, tlen, i, j, netstart, matched;
+   char *buspos, *busend, *busptr;
+
+   if (pin_is_bus(blab, thisinst) == FALSE) return NULL;
+   if (subnets == NULL) {
+      /* This happens on the first pass only */
+      subnets = (Genericlist *)malloc(sizeof(Genericlist));
+      subnets->net.list = (buslist *)malloc(sizeof(buslist));
+   }
+   subnets->subnets = 0;
+
+   tptr = textprint(blab->string, thisinst);
+   tlen = strlen(tptr) + 1;
+   buspos = strchr(tptr, areastruct.buschar);
+
+   /* The notation "(...)" with NO TEXT preceding the opening bus	*/
+   /* delimiter, is assumed to represent a numerical pin range.  So	*/
+   /* instead of generating labels, e.g., "(1)", "(2)", etc., we	*/
+   /* generate	labels "1", "2", etc.					*/
+
+   if (buspos == NULL) {
+      Fprintf(stderr, "Error:  Bus specification has no start delimiter!\n");
+      goto doneBus;
+   }
+
+   netstart = (netlist->subnets == 0) ? netlist->net.id : 0;
+
+   bpos = (int)(buspos - tptr);
+   busend = find_delimiter(buspos);
+
+   if (busend == NULL) {
+      Fprintf(stderr, "Error:  Bus specification has no end delimiter!\n");
+      goto doneBus;
+   }
+
+   /* Find the range of each bus */
+
+   matched = 0;
+   istart = -1;
+   for (busptr = buspos + 1; busptr < busend; busptr++) {
+      if (sscanf(busptr, "%d", &iend) == 0) break;
+      while ((*busptr != ':') && (*busptr != '-') && (*busptr != ',')
+		&& (*busptr != *busend))
+	 busptr++;
+      if ((*busptr == ':') || (*busptr == '-')) 	/* numerical range */
+	 istart = iend;
+      else {
+	 if (istart < 0) istart = iend;
+	 i = istart;
+	 while (1) {
+
+	    /* Create a new list entry for this subnet number */
+
+	    subnets->subnets++;
+	    subnets->net.list = (buslist *)realloc(subnets->net.list,
+			subnets->subnets * sizeof(buslist));
+
+	    sbus = subnets->net.list + subnets->subnets - 1;
+	    sbus->subnetid = i;
+	    if (netstart > 0) {
+	       sbus->netid = netstart++;
+	       matched++;
+	    }
+	    else {
+	       /* Net ID is the net ID for the matching subnet of netlist */
+	       for (j = 0; j < netlist->subnets; j++) {
+		  jbus = netlist->net.list + j;
+		  if (jbus->subnetid == i) {
+		     matched++;
+		     sbus->netid = jbus->netid;
+		     break;
+		  }
+	       }
+	       /* Insert a net ID of zero if it can't be found */
+	       if (j == netlist->subnets) {
+		  sbus->netid = 0;
+	       }
+	    }
+
+	    if (i == iend) break;
+	    else if (istart > iend) i--;
+	    else i++;
+	 }
+	 istart = -1;
+      }
+   }
+
+doneBus:
+   free(tptr);
+   return (matched == 0) ? NULL : subnets;
+}
+
 /*----------------------------------------------------------------------*/
 /* Test equivalence of two label strings (relaxed constraints)		*/
 /*    Like stringcomp(), but ignores "superficial" differences such as  */
 /*    color and font (unless one of the fonts is Symbol), scale,	*/
 /*    underlining, tabbing, and kerning.				*/
+/*									*/
+/* Return 0 if matching.  Return 1 if not matching.			*/
+/*									*/
+/* For bus notation, ignore everything inside the bus delimiters.	*/
+/* The calling routine must determine if the labels being compared	*/
+/* have matching subnet ranges.  Note that as written, this does not	*/
+/* check any text occurring after the opening bus delimiter!  Thus,	*/
+/* "mynet(1:2)" and "mynet(3:4)" and "mynet(1)_alt" are all considered	*/
+/* exact matches in bus notation.					*/
 /*----------------------------------------------------------------------*/
 
 int stringcomprelaxed(stringpart *string1, stringpart *string2,
 			objinstptr thisinst)
 {
    stringpart *strptr1 = string1, *strptr2 = string2;
-   Boolean font1 = False, font2 = False;
+   Boolean font1 = False, font2 = False, inbus_match = TRUE;
+   int in_bus = 0;
+   char *buspos;
+   int bpos;
 
    if (strptr1->type == FONT_NAME)
       font1 = issymbolfont(strptr1->data.font);
@@ -672,7 +961,47 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
       else {
          switch (strptr1->type) {
 	    case TEXT_STRING:
-	       if (strcmp(strptr1->data.string, strptr2->data.string)) return 1;
+	       if (in_bus == 1) {
+		  char matchchar = areastruct.buschar;
+		  switch (areastruct.buschar) {
+		     case '(': matchchar = ')'; break;
+		     case '[': matchchar = ']'; break;
+		     case '{': matchchar = '}'; break;
+		     case '<': matchchar = '>'; break;
+		  }
+		  buspos = strchr(strptr1->data.string, matchchar);
+		  if (buspos != NULL) {
+		     bpos = (int)(buspos - (char *)(strptr1->data.string));
+		     if (strlen(strptr2->data.string) > bpos) {
+		        if (!strcmp(strptr1->data.string + bpos,
+			  	strptr2->data.string + bpos)) {
+		           in_bus = 2;
+		           break;
+			}
+		     }
+		     return 1;
+		  }
+		  if (inbus_match == TRUE)
+		     if (strcmp(strptr1->data.string, strptr2->data.string))
+			 inbus_match = FALSE;
+	       }
+	       else if (!strcmp(strptr1->data.string, strptr2->data.string))
+		  break;
+
+	       /* To be a matching bus, the strings match everywhere	*/
+	       /* except between the bus notation delimiters (default	*/
+	       /* "()".							*/
+		
+	       buspos = strchr(strptr1->data.string, areastruct.buschar);
+	       if (buspos != NULL) {
+
+		  bpos = (int)(buspos - (char *)(strptr1->data.string)) + 1;
+		  if (!strncmp(strptr1->data.string, strptr2->data.string, bpos)) {
+		     in_bus = 1;
+		     break;
+		  }
+	       }
+	       return 1;  /* strings did not match, exactly or as bus notation */
 	       break;
 	    case OVERLINE:
 	       if (strptr1->type != strptr2->type) return 1;
@@ -685,8 +1014,12 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
 
    /* One string continues after the other ends. . . */ 
    if (strptr1 != NULL || strptr2 != NULL) return 1;
+
+   /* Treat no closing bus delimiter as a non-bus string */
+   else if ((in_bus == 1) && (inbus_match == FALSE)) return 1;
    return 0;
 }
+
 /*----------------------------------------------------------------------*/
 /* Find the number of parts in a string	(excluding parameter contents)	*/
 /*----------------------------------------------------------------------*/
@@ -789,7 +1122,8 @@ stringpart *stringcopyall(stringpart *string, objinstptr thisinst)
 
 stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 {
-   stringpart *strptr, *newpart, *curend = NULL, *rettop, *curtop, *savend;
+   stringpart *strptr, *newpart, *curend = NULL;
+   stringpart *rettop, *curtop, *savend = NULL;
    char *key = NULL;
    oparamptr pparam;
    Boolean need_free;
