@@ -92,9 +92,8 @@ char	 _STR2[250];  /* Specifically for text returned from the popup prompt */
 char	 _STR[150];          /* Generic multipurpose string */
 short	 popups;	             /* total number of popup widgets on the screen */
 
-xcWidget	 message1, message2, message3, top, toolbar, overlay = NULL;
-
 #ifndef TCL_WRAPPER
+xcWidget   message1, message2, message3, top, toolbar, overlay = NULL;
 xcWidget   menuwidgets[MaxMenuWidgets];
 #endif
 
@@ -131,10 +130,11 @@ short beeper;
 short fontcount;
 fontinfo *fonts;
 
-extern short eventmode, textpos, help_up;
+extern short eventmode, textpos;
 extern float version;
 
 #ifndef TCL_WRAPPER
+extern short help_up;
 extern menustruct TopButtons[];
 #ifdef HAVE_XPM
 extern toolbarstruct ToolBar[];
@@ -181,7 +181,7 @@ static XtResource resources[] = {
   { "infolabelcolor", "InfoLabelColor", XtRPixel, sizeof(Pixel),
       XtOffset(ApplicationDataPtr, infocolor), XtRString, "SeaGreen"},
   { "ratsnestcolor", "RatsNestColor", XtRPixel, sizeof(Pixel),
-      XtOffset(ApplicationDataPtr, ratsnestcolor), XtRString, "tan4"},
+      XtOffset(ApplicationDataPtr, ratsnestcolor), XtRString, "Tan4"},
 
   /* non-schematic layout colors */
 
@@ -656,6 +656,10 @@ void quit(xcWidget w, caddr_t nulldata)
 
    exit_gs();
 
+   /* exit ngspice if the simulator is still running */
+
+   exit_spice();
+
    /* remove any temporary files created for background rendering */
 
    for (i = 0; i < xobjs.pages; i++) {
@@ -1625,36 +1629,10 @@ xcTimeOutProc clrmessage(caddr_t clientdata, xcIntervalId *id)
    }
 }
 
+#ifndef TCL_WRAPPER
+
 /*------------------------------------------------------------------------------*/
 
-#ifdef TCL_WRAPPER
-
-/* These should be replaced with script versions */
-
-void W0printf(char *string, char *window)
-{
-    char _STR3[300];
-    sprintf(_STR3, ".xcircuit.%s config -text {%s}", window, string);
-    Tcl_Eval(xcinterp, _STR3);
-}
-
-void W1printf(char *string)
-{
-    W0printf(string, "menubar.message");
-}
-
-void W2printf(char *string)
-{
-    W0printf(string, "infobar.message1");
-}
-
-void Wprintf(char *string)
-{
-    W0printf(string, "infobar.message2");
-}
-
-#else
-
 void W1printf(char *string)
 {
    Arg	wargs[1];
@@ -1843,6 +1821,21 @@ void getcommand(xcWidget cmdw, caddr_t clientdata, caddr_t calldata)
    XtUnmanageChild(cmdw);
 }
 
+#endif /* !TCL_WRAPPER */
+
+#ifdef TCL_WRAPPER
+
+/*------------------------------------------------------------------------------*/
+/* "docommand" de-iconifies the TCL console.					*/
+/*------------------------------------------------------------------------------*/
+
+void docommand()
+{
+   Tcl_Eval(xcinterp, "catch xcircuit::raiseconsole");
+}
+
+#else  /* !TCL_WRAPPER */
+
 /*------------------------------------------------------------------------------*/
 /* "docommand" overlays the message3 widget temporarily with a TextEdit widget. */
 /*------------------------------------------------------------------------------*/
@@ -2002,6 +1995,7 @@ int xc_getlayoutcolor(int cidx)
    XQueryColor(dpy, cmap, &loccolor);
 
    locidx = rgb_alloccolor(loccolor.red, loccolor.green, loccolor.blue);
+   return locidx;
 }
 
 /*-------------------------------------------------------------------------*/
@@ -2154,6 +2148,8 @@ void makecursors()
    XRecolorCursor(dpy, TEXTPTR, &fgcolor, &bgcolor);
 }
 
+#ifndef TCL_WRAPPER
+
 /*----------------------------------------------------------------------*/
 /* Event handler for input focus					*/
 /*----------------------------------------------------------------------*/
@@ -2176,20 +2172,27 @@ void mappinghandler(xcWidget w, caddr_t clientdata, XEvent *event)
    }
 }
 
+#endif
+
 /*----------------------------------------------------------------------*/
 
 void clientmessagehandler(xcWidget w, caddr_t clientdata, XEvent *event)
 {
    if (!xcIsRealized(w)) return;
-   if (areastruct.mapped == True) {
-      /* Fprintf(stderr, "Forcing input focus\n"); */
-      XSetInputFocus(dpy, xcWindow(w), RevertToPointerRoot, CurrentTime);  
+
+   if (event->type == ClientMessage) {
+      if (render_client(event) == False)
+	 return;
+
+#ifdef INPUT_FOCUS
+      if (areastruct.mapped == True) {
+         /* Fprintf(stderr, "Forcing input focus\n"); */
+         XSetInputFocus(dpy, xcWindow(w), RevertToPointerRoot, CurrentTime);  
+      }
+#endif
    }
 }
 
-#endif
-
-#ifndef TCL_WRAPPER
 /*----------------------------------------------------------------------*/
 /* Event handler for WM_DELETE_WINDOW message.                          */
 /*----------------------------------------------------------------------*/
@@ -2206,7 +2209,8 @@ void delwin(xcWidget w, popupstruct *bstruct, XClientMessageEvent *event)
          destroypopup(w, bstruct, NULL);
    }
 }
-#endif
+
+#endif /* !TCL_WRAPPER */
 
 /*----------------------------------------------------------------------*/
 /* Preparatory initialization (to be run before setting up the GUI)	*/
@@ -2292,7 +2296,7 @@ void pre_initialize()
    areastruct.gridon = True;
    areastruct.center = True;
    areastruct.bboxon = False;
-   areastruct.filter = SEL_ANY;
+   areastruct.filter = ALL_TYPES;
    areastruct.filefilter = True;
    areastruct.editinplace = True;
    areastruct.selects = 0;
@@ -2306,8 +2310,8 @@ void pre_initialize()
    initmem(areastruct.editstack);
    areastruct.pinpointon = False;
    areastruct.schemon = False;
+   areastruct.buschar = '(';	/* Vector notation for buses */
 
-   /* xobjs.numlibs = LIBS - LIBRARY; */
    xobjs.numlibs = LIBS - LIBRARY - 1;
    xobjs.fontlib.number = 0;
    xobjs.userlibs = (Library *) malloc(xobjs.numlibs * sizeof(Library));
@@ -2688,9 +2692,10 @@ void GUI_init(int argc, char *argv[])
 #ifdef INPUT_FOCUS
    XtAddEventHandler(top, SubstructureNotifyMask,
 	TRUE, (XtEventHandler)mappinghandler, NULL);
+#endif
+
    XtAddEventHandler(top, NoEventMask, TRUE,
 	(XtEventHandler)clientmessagehandler, NULL);
-#endif
 
    /*---------------------------------------------------*/
    /* Define basic display variables 			*/
