@@ -32,18 +32,11 @@ typedef unsigned long	u_long;
   #define calloc(a,b) Tcl_Alloc(a * b)
   #define free(a) Tcl_Free((char *)(a))
   #define realloc(a,b) Tcl_Realloc((char *)(a), b)
+  #undef strdup
+  #define strdup Tcl_Strdup
+  extern char *Tcl_Strdup(const char *);
 #endif
 
-/*----------------------------------------------------------------------*/
-/* Definition shortcuts 						*/
-/*----------------------------------------------------------------------*/
-
-#define XtnSetArg(a,b)  	XtSetArg(wargs[n], a, b); n++
-#define abs(a)			((a) < 0 ? -(a) : (a))
-#define sign(a)			((a) <= 0 ? -1 : 1)
-#define max(a,b)		((a) < (b) ? (b) : (a))
-#define min(a,b)		((a) < (b) ? (a) : (b))
-
 /*----------------------------------------------------------------------*/
 /* Deal with 32/64 bit processors based on autoconf results.		*/
 /*----------------------------------------------------------------------*/
@@ -58,6 +51,44 @@ typedef u_long pointertype;
 ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #endif
 
+/*----------------------------------------------------------------------*/
+/* Basic element types 							*/
+/*									*/
+/* These values determine how a genericptr should be cast into one of	*/
+/* labelptr, polyptr, etc.  To query the element type, use the macros	*/
+/* ELEMENTTYPE(), IS_LABEL(), IS_POLYGON(), etc.  These macros mask the	*/
+/* bit field.  Note that an element marked with REMOVE_TAG will not be	*/
+/* recognized as any valid element, but otherwise, querying the type is	*/
+/* transparent to other bit settings (such as NETLIST_INVALID).		*/
+/*----------------------------------------------------------------------*/
+
+/* Single-bit flags */
+#define	OBJINST		1
+#define LABEL		2
+#define POLYGON		4
+#define ARC		8
+#define SPLINE  	16
+#define PATH		32
+
+#define REMOVE_TAG	64	/* element to be removed from netlist	*/
+#define NETLIST_INVALID 128	/* this element invalidates the netlist	*/
+#define SELECT_HIDE	256	/* this element cannot be selected	*/
+#define DRAW_HIDE	512	/* this element is not drawn.		*/
+
+/* Bit fields */
+#define ALL_TYPES 	(OBJINST | LABEL | POLYGON | ARC | SPLINE | PATH)
+#define VALID_TYPES 	(REMOVE_TAG | ALL_TYPES)
+
+/*----------------------------------------------------------------------*/
+/* Definition shortcuts 						*/
+/*----------------------------------------------------------------------*/
+
+#define XtnSetArg(a,b)  	XtSetArg(wargs[n], a, b); n++
+#define abs(a)			((a) < 0 ? -(a) : (a))
+#define sign(a)			((a) <= 0 ? -1 : 1)
+#define max(a,b)		((a) < (b) ? (b) : (a))
+#define min(a,b)		((a) < (b) ? (a) : (b))
+
 /*----------------------------------------------------------------------*/
 /* Lengthier define constructs						*/
 /*----------------------------------------------------------------------*/
@@ -69,14 +100,25 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 #define PLIST_INCR(a) \
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
-#define SELECTTYPE(a)   (*(topobject->plist + *(a)))->type
+#define SELECTTYPE(a)   ((*(topobject->plist + *(a)))->type & ALL_TYPES)
 #define SELTOCOLOR(a)	(*(topobject->plist + *(a)))->color
 #define GSELTOCOLOR(b, a) (*((b)->plist + *(a)))->color
 #define ENDPART		topobject->plist + topobject->parts
 #define EDITPART	topobject->plist + areastruct.editpart
 
-/* conversions from generic to specific types */
-/* specifically, from type (generic *) to type (polyptr), etc. */
+/* Type checks (argument "a" is type genericptr) */
+
+#define ELEMENTTYPE(a)	((a)->type & VALID_TYPES)
+
+#define IS_POLYGON(a)   (ELEMENTTYPE(a) == POLYGON)
+#define IS_LABEL(a)     (ELEMENTTYPE(a) == LABEL)
+#define IS_OBJINST(a)   (ELEMENTTYPE(a) == OBJINST)
+#define IS_ARC(a)       (ELEMENTTYPE(a) == ARC)
+#define IS_SPLINE(a)    (ELEMENTTYPE(a) == SPLINE)
+#define IS_PATH(a)      (ELEMENTTYPE(a) == PATH)
+
+/* Conversions from generic to specific types */
+/* specifically, from type (genericptr *) to type (polyptr), etc. */
 
 #define TOPOLY(a)	(*((polyptr *)(a)))
 #define TOLABEL(a)	(*((labelptr *)(a)))
@@ -111,7 +153,7 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	PLIST_INCR(b); \
 	a = (objinstptr *)b->plist + b->parts; \
 	*a = (objinstptr) malloc(sizeof(objinst)); \
-	(*a)->type = OBJECT
+	(*a)->type = OBJINST
 #define NEW_ARC(a,b) \
 	PLIST_INCR(b); \
 	a = (arcptr *)b->plist + b->parts; \
@@ -128,11 +170,21 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	*a = (pathptr) malloc(sizeof(path)); \
 	(*a)->type = PATH
 
-/* selection from top-level object */
+/*----------------------------------------------------------------------*/
+/* Undo mechanism definitions						*/
+/*----------------------------------------------------------------------*/
 
-#define objectselect(a)	 genobjectselect(a, topobject, areastruct.topinstance)
+enum UNDO_MODES {UNDO_DONE = 0, UNDO_MORE};
+
+/*----------------------------------------------------------------------*/
+/* selection from top-level object 					*/
+/*----------------------------------------------------------------------*/
+
+#define select_element(a) genselectelement(a, UNDO_MORE, topobject,\
+	areastruct.topinstance)
+#define select_add_element(a) genselectelement(a, UNDO_DONE, topobject,\
+	areastruct.topinstance)
 #define easydraw(a, b)	 geneasydraw(a, b, topobject, areastruct.topinstance)
-#define drawselected(a)	 gendrawselected(a, topobject, areastruct.topinstance)
 
 /*----------------------------------------------------------------------*/
 
@@ -200,7 +252,7 @@ typedef struct {
 /* Event mode definitions (state of drawing area)			*/
 /*----------------------------------------------------------------------*/
 
-enum editmode {NORMAL_MODE = 0, 
+enum editmode {NORMAL_MODE = 0, UNDO_MODE,
 PENDING_MODE, PRESS_MODE, DELETE_MODE, COPY_MODE, COPY2_MODE, ROTATE_MODE,
 PUSH_MODE, EDIT_MODE, SELAREA_MODE, SELAREA2_MODE, DESEL_MODE, PAN_MODE,
 CATPAN_MODE, CATALOG_MODE, CATTEXT_MODE, FONTCAT_MODE, FONTCAT2_MODE,
@@ -258,8 +310,8 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define FONT_SCALE	14  /* font scale change; data = scale		*/
 #define FONT_COLOR	15  /* font color change; data = color		*/
 #define KERN		16  /* set new kern values; data = kern x, y	*/
-#define PARAM_START	17  /* bounds a parameter; data = param #	*/
-#define PARAM_END	18  /* bounds a parameter; data = param #	*/
+#define PARAM_START	17  /* bounds a parameter; data = param key	*/
+#define PARAM_END	18  /* bounds a parameter; no data 		*/
 
 #define SPECIAL		63  /* used only when called from menu		*/
 #define NULL_TYPE	255 /* used as a placeholder			*/
@@ -398,20 +450,6 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 typedef XPoint* pointlist;
 typedef XfPoint* fpointlist;
 
-/*----------------------------------------------------------------------*/
-/* select types 							*/
-/*----------------------------------------------------------------------*/
-
-#define	OBJECT	1
-#define LABEL	2
-#define POLYGON	4
-#define ARC	8
-#define SPLINE  16
-#define PATH	32
-#define SEL_ANY	63  /* for selection mechanism only */
-
-#define REMOVE_TAG 64  /* add this value to objects to be removed */
-
 /*----------------------------------------------------------------------*/
 /* Allowed parameterization types					*/
 /*----------------------------------------------------------------------*/
@@ -501,7 +539,7 @@ typedef struct _eparam {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;		/* type is LABEL, POLYGON, etc., from below */
+   u_short	type;		/* type is LABEL, POLYGON, etc., from below */
    int		color;
    eparamptr	passed;
 } generic, *genericptr;		/* (convenience function for retypecasting) */
@@ -522,7 +560,7 @@ typedef struct {
 typedef struct _xcobject *objectptr;
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    XPoint	position;
@@ -569,7 +607,7 @@ typedef struct _liblist {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    XPoint	position;
@@ -585,7 +623,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
@@ -599,7 +637,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
@@ -614,7 +652,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
@@ -634,7 +672,7 @@ typedef struct {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   u_char	type;
+   u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
    u_short	style;
@@ -647,14 +685,38 @@ typedef struct {
 /* Netlist structures for schematic capture				*/
 /*----------------------------------------------------------------------*/
 
+/* Structure to hold net and subnet IDs for a bus */
+
+typedef struct {
+   int netid;
+   int subnetid;
+} buslist;
+
+/* Structure mimicking the top part of a Polylist or Labellist	*/
+/* when we just want the netlist information and don't care	*/
+/* which one we're looking at.					*/
+
+typedef struct {
+   union {
+      int id;
+      buslist *list;
+   } net;
+   int subnets;
+} Genericlist; 
+
 /* Linked polygon list */
 
 typedef struct _Polylist *PolylistPtr;
 typedef struct _Polylist
 {
-   objectptr cschem;	/* Schematic containing the polygon */
+   union {
+      int  id;		/* A single net ID, if subnets == 0	*/
+      buslist *list;	/* List of net and subnet IDs for a bus	*/
+   } net;
+   int subnets;		/* Number of subnets; 0 if no subnets	*/
+   objectptr cschem;	/* Schematic containing the polygon	*/
    polyptr poly;
-   PolylistPtr next;	/* Next polygon in the linked list */
+   PolylistPtr next;	/* Next polygon in the linked list	*/
 } Polylist;
 
 /* Linked label list */
@@ -662,22 +724,26 @@ typedef struct _Polylist
 typedef struct _Labellist *LabellistPtr;
 typedef struct _Labellist
 {
-   objectptr cschem;	/* Schematic containing the label */
+   union {
+      int  id;		/* A single net ID, if subnets == 0	*/
+      buslist *list;	/* List of net and subnet IDs for a bus	*/
+   } net;
+   int subnets;		/* Number of subnets; 0 if no subnets	*/
+   objectptr cschem;	/* Schematic containing the label	*/
+   objinstptr cinst;	/* Specific label instance, if applicable */
    labelptr label;
    LabellistPtr next;
 } Labellist;
 
-/* List of object's networks */
+/* List of object's networks by (flattened) name */
 
-typedef struct _Netlist *NetlistPtr;
-typedef struct _Netlist
+typedef struct _Netname *NetnamePtr;
+typedef struct _Netname
 {
    int netid;
-   PolylistPtr polygons;
-   LabellistPtr labels;
    stringpart *localpin;
-   NetlistPtr next;
-} Netlist;
+   NetnamePtr next;
+} Netname;
 
 /* List of object's I/O ports */
 
@@ -695,7 +761,7 @@ typedef struct _Portlist
 typedef struct _Calllist *CalllistPtr;
 typedef struct _Calllist
 {
-   objectptr   cschem;		/* Schematic page containing the instance */
+   objectptr   cschem;		/* Schematic containing the instance called */
    objinstptr  callinst;	/* Instance called */
    objectptr   callobj;		/* Object of instance called */
    int         devindex;	/* if non-zero, index of device in netlist */
@@ -728,7 +794,7 @@ struct Ptab {
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   int		netid;
+   Genericlist	*netlist;
    objinstptr	thisinst;
 } Highlight;
 
@@ -754,9 +820,12 @@ typedef struct _xcobject {
    char		*devname;	/* if non-null, name of device in netlist */
    Boolean	valid;		/* Is current netlist valid? */
    Boolean	traversed;	/* Flag to indicate object was processed */
-   NetlistPtr   netlist;
-   PortlistPtr  portlist;
-   CalllistPtr  calllist;
+   LabellistPtr	labels;		/* Netlist pins	 */
+   PolylistPtr	polygons;	/* Netlist wires */
+   PortlistPtr  ports;		/* Netlist ports */
+   CalllistPtr  calls;		/* Netlist subcircuits and connections */
+   NetnamePtr   netnames;	/* Local names for flattening */
+				/* (this probably shouldn't be here. . .) */
 } object;
 
 /*----------------------------------------------------------------------*/
@@ -1011,7 +1080,7 @@ enum {
    XCF_Double_Snap, XCF_Halve_Snap, XCF_Pan_Left, XCF_Pan_Right, XCF_Pan_Up,
    XCF_Pan_Down, XCF_Write, XCF_Rotate, XCF_Flip_X, XCF_Flip_Y, XCF_Snap,
    XCF_SnapTo, XCF_Pop, XCF_Push, XCF_Delete, XCF_Select, XCF_Box, XCF_Arc,
-   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Join, XCF_Unjoin, XCF_Spline,
+   XCF_Text, XCF_Exchange, XCF_Copy, XCF_Move, XCF_Join, XCF_Unjoin, XCF_Spline,
    XCF_Edit, XCF_Undo, XCF_Redo, XCF_Select_Save, XCF_Unselect, XCF_Dashed,
    XCF_Dotted, XCF_Solid, XCF_Prompt, XCF_Dot, XCF_Wire, XCF_Start,
    XCF_Finish, XCF_Cancel, XCF_Nothing, XCF_Exit,
@@ -1080,6 +1149,7 @@ typedef struct {
    Boolean	invert;
    Boolean	mapped;		/* indicates if window is drawable */
    Boolean	filefilter;	/* Is the file list filtered? */
+   char		buschar;	/* Character indicating vector notation */
    Boolean	editinplace;
    Boolean	schemon;
    Boolean	pinpointon;
@@ -1109,6 +1179,7 @@ typedef struct _undostack {
    Undoptr next;	/* next record in undo stack */
    Undoptr last;	/* double-linked for "redo" function */
    u_int type;		/* type of event */
+   short idx;		/* counter for undo event */
    objinstptr thisinst;	/* instance of object in which event occurred */
    int idata;		/* simple undedicated integer datum */
    char *undodata;	/* free space to be malloc'd */
