@@ -52,6 +52,7 @@ extern int xctcl_rotate();
 extern int xctcl_flip();
 extern int xctcl_set();
 extern int xctcl_page();
+extern int xctcl_spice();
 extern int xctcl_zoom();
 extern int xctcl_pan();
 extern int xctcl_element();
@@ -120,6 +121,7 @@ static cmdstruct xc_commands[] =
    {"netlist", (void *)xctcl_netlist},
    {"symbol", (void *)xctcl_symschem},
    {"schematic", (void *)xctcl_symschem},
+   {"spice", (void *)xctcl_spice},
    {"", NULL}  /* sentinel */
 };
 
@@ -202,6 +204,11 @@ Xcircuit_Init(Tcl_Interp *interp)
    sprintf(version_string, "%g", PROG_VERSION);
    Tcl_SetVar(interp, "XCIRCUIT_VERSION", version_string, TCL_GLOBAL_ONLY);
 
+#ifdef ASG
+   /* Set a variable in Tcl indicating that the ASG module is compiled in */
+   Tcl_SetVar(interp, "XCIRCUIT_ASG", "1", TCL_GLOBAL_ONLY);
+#endif
+
    /* Export the namespace commands */
 
    Tcl_Eval(interp, "namespace eval xcircuit namespace export *");
